@@ -6,60 +6,54 @@ from pyinfra import __version__, state
 
 from .exceptions import PyinfraError
 
-config_defaults = {
+
+class ConfigDefaults:
     # % of hosts which have to fail for all operations to stop
-    "FAIL_PERCENT": None,
+    FAIL_PERCENT = None
     # Seconds to timeout SSH connections
-    "CONNECT_TIMEOUT": 10,
+    CONNECT_TIMEOUT = 10
     # Temporary directory (on the remote side) to use for caching any files/downloads
-    "TEMP_DIR": "/tmp",
+    TEMP_DIR = "/tmp"
     # Gevent pool size (defaults to #of target hosts)
-    "PARALLEL": None,
+    PARALLEL = 0
     # Specify the required pyinfra version (using PEP 440 setuptools specifier)
-    "REQUIRE_PYINFRA_VERSION": None,
+    REQUIRE_PYINFRA_VERSION = None
     # Specify any required packages (either using PEP 440 or a requirements file)
-    # Note: this can also include pyinfra, potentially replacing REQUIRE_PYINFRA_VERSION
-    "REQUIRE_PACKAGES": None,
+    # Note: this can also include pyinfra potentially replacing REQUIRE_PYINFRA_VERSION
+    REQUIRE_PACKAGES = None
     # All these can be overridden inside individual operation calls:
     # Switch to this user (from ssh_user) using su before executing operations
-    "SU_USER": None,
-    "USE_SU_LOGIN": False,
-    "SU_SHELL": None,
-    "PRESERVE_SU_ENV": False,
+    SU_USER = None
+    USE_SU_LOGIN = False
+    SU_SHELL = None
+    PRESERVE_SU_ENV = False
     # Use sudo and optional user
-    "SUDO": False,
-    "SUDO_USER": None,
-    "PRESERVE_SUDO_ENV": False,
-    "USE_SUDO_LOGIN": False,
-    "USE_SUDO_PASSWORD": False,
+    SUDO = False
+    SUDO_USER = None
+    PRESERVE_SUDO_ENV = False
+    USE_SUDO_LOGIN = False
+    USE_SUDO_PASSWORD = False
     # Use doas and optional user
-    "DOAS": False,
-    "DOAS_USER": None,
-    # Use doas and optional user
-    "DOAS": False,
-    "DOAS_USER": None,
-    # Only show errors, but don't count as failure
-    "IGNORE_ERRORS": False,
+    DOAS = False
+    DOAS_USER = None
+    # Only show errors but don't count as failure
+    IGNORE_ERRORS = False
     # Shell to use to execute commands
-    "SHELL": "sh",
-}
+    SHELL = "sh"
 
 
-def check_pyinfra_version(version):
+config_defaults = {key: value for key, value in ConfigDefaults.__dict__.items() if key.isupper()}
+
+
+def check_pyinfra_version(version: str):
     if not version:
         return
-
     running_version = parse_version(__version__)
-    required_versions = Requirement.parse(
-        "pyinfra{0}".format(version),
-    )
+    required_versions = Requirement.parse("pyinfra{0}".format(version))
 
-    if running_version not in required_versions:
+    if not required_versions.specifier.contains(running_version):
         raise PyinfraError(
-            ("pyinfra version requirement not met " "(requires {0}, running {1})").format(
-                version,
-                __version__,
-            ),
+            f"pyinfra version requirement not met (requires {version}, running {__version__})"
         )
 
 
@@ -70,7 +64,7 @@ def check_require_packages(requirements_config):
     if isinstance(requirements_config, (list, tuple)):
         requirements = requirements_config
     else:
-        with open(path.join(state.cwd, requirements_config)) as f:
+        with open(path.join(state.cwd, requirements_config), encoding="utf-8") as f:
             requirements = [line.split("#egg=")[-1] for line in f.read().splitlines()]
 
     try:
@@ -90,13 +84,11 @@ config_checkers = {
 }
 
 
-class Config(object):
+class Config(ConfigDefaults):
     """
     The default/base configuration options for a pyinfra deploy.
     """
 
-    state = None
-
     def __init__(self, **kwargs):
         # Always apply some env
         env = kwargs.pop("ENV", {})
@@ -128,5 +120,5 @@ class Config(object):
     def reset_locked_state(self):
         self.set_current_state(self._locked_config)
 
-    def copy(self):
+    def copy(self) -> "Config":
         return Config(**dict(self.get_current_state()))
