@@ -2,12 +2,16 @@
 The pyinfra facts API. Facts enable pyinfra to collect remote server state which
 is used to "diff" with the desired state, producing the final commands required
 for a deploy.
+
+Note that the facts API does *not* use the global currently in context host so
+it's possible to call facts on hosts out of context (ie give me the IP of this
+other host B while I operate on this host A).
 """
 
 import re
 from inspect import getcallargs
 from socket import error as socket_error, timeout as timeout_error
-from typing import Optional, Type
+from typing import TYPE_CHECKING, Any, Callable, Dict, List, Optional, Type, Union
 
 import click
 import gevent
@@ -29,7 +33,11 @@ from pyinfra.progress import progress_spinner
 
 from .arguments import get_executor_kwarg_keys
 
-SUDO_REGEX = r"^sudo: unknown user:"
+if TYPE_CHECKING:
+    from pyinfra.api.host import Host
+    from pyinfra.api.state import State
+
+SUDO_REGEX = r"^sudo: unknown user"
 SU_REGEXES = (
     r"^su: user .+ does not exist",
     r"^su: unknown login",
@@ -37,18 +45,23 @@ SU_REGEXES = (
 
 
 class FactNameMeta(type):
-    def __init__(cls, name, bases, attrs):
+    def __init__(cls, name: str, bases, attrs, **kwargs):
+        super().__init__(name, bases, attrs, **kwargs)
         module_name = cls.__module__.replace("pyinfra.facts.", "")
         cls.name = f"{module_name}.{cls.__name__}"
 
 
-class FactBase(object, metaclass=FactNameMeta):
-    abstract = True
+class FactBase(metaclass=FactNameMeta):
+    name: str
+
+    abstract: bool = True
 
     shell_executable: Optional[str] = None
 
     requires_command: Optional[str] = None
 
+    command: Union[str, Callable]
+
     @staticmethod
     def default():
         """
@@ -63,11 +76,15 @@ class FactBase(object, metaclass=FactNameMeta):
         return {arg: self.process([output[i]]) for i, arg in enumerate(args)}
 
 
-class ShortFactBase(object, metaclass=FactNameMeta):
+class ShortFactBase(metaclass=FactNameMeta):
     fact: Type[FactBase]
 
+    @staticmethod
+    def process_data(data):
+        return data
 
-def get_short_facts(state, host, short_fact, **kwargs):
+
+def get_short_facts(state: "State", host: "Host", short_fact, **kwargs):
     fact_data = get_fact(state, host, short_fact.fact, **kwargs)
     return short_fact().process_data(fact_data)
 
@@ -79,18 +96,25 @@ def _make_command(command_attribute, host_args):
     return command_attribute
 
 
-def _get_executor_kwargs(state, host, override_kwargs=None, override_kwarg_keys=None):
+def _get_executor_kwargs(
+    state: "State",
+    host: "Host",
+    override_kwargs: Optional[Dict[str, Any]] = None,
+    override_kwarg_keys: Optional[List[str]] = None,
+):
     if override_kwargs is None:
         override_kwargs = {}
+    if override_kwarg_keys is None:
         override_kwarg_keys = []
 
+    # Use the current operation global kwargs, or generate defaults
+    global_kwargs = host.current_op_global_kwargs
+    if not global_kwargs:
+        global_kwargs, _ = pop_global_arguments({}, state, host)
+
     # Apply any current op kwargs that *weren't* found in the overrides
     override_kwargs.update(
-        {
-            key: value
-            for key, value in (host.current_op_global_kwargs or {}).items()
-            if key not in override_kwarg_keys
-        },
+        {key: value for key, value in global_kwargs.items() if key not in override_kwarg_keys},
     )
 
     return {
@@ -98,7 +122,41 @@ def _get_executor_kwargs(state, host, override_kwargs=None, override_kwarg_keys=
     }
 
 
-def get_facts(state, *args, **kwargs):
+def _handle_fact_kwargs(state, host, cls, args, kwargs):
+    args = args or []
+    kwargs = kwargs or {}
+
+    # TODO: this is here to avoid popping stuff accidentally, this is horrible! Change the
+    # pop function to return the clean kwargs to avoid the indirect mutation.
+    kwargs = kwargs.copy()
+
+    # Get the defaults *and* overrides by popping from kwargs, executor kwargs passed
+    # into get_fact override everything else (applied below).
+    override_kwargs, override_kwarg_keys = pop_global_arguments(
+        kwargs,
+        state=state,
+        host=host,
+        keys_to_check=get_executor_kwarg_keys(),
+    )
+
+    executor_kwargs = _get_executor_kwargs(
+        state,
+        host,
+        override_kwargs=override_kwargs,
+        override_kwarg_keys=override_kwarg_keys,
+    )
+
+    fact_kwargs = {}
+
+    if args or kwargs:
+        assert not isinstance(cls.command, str)
+        # Merges args & kwargs into a single kwargs dictionary
+        fact_kwargs = getcallargs(cls().command, *args, **kwargs)
+
+    return fact_kwargs, executor_kwargs
+
+
+def get_facts(state: "State", *args, **kwargs):
     def get_fact_with_context(state, host, *args, **kwargs):
         with ctx_state.use(state):
             with ctx_host.use(host):
@@ -121,15 +179,15 @@ def get_facts(state, *args, **kwargs):
 
 
 def get_fact(
-    state,
-    host,
-    cls,
-    args=None,
-    kwargs=None,
-    ensure_hosts=None,
-    apply_failed_hosts=True,
-    fact_hash=None,
-    use_cache=True,
+    state: "State",
+    host: "Host",
+    cls: Type[FactBase],
+    args: Optional[Any] = None,
+    kwargs: Optional[Any] = None,
+    ensure_hosts: Optional[Any] = None,
+    apply_failed_hosts: bool = True,
+    fact_hash: Optional[Any] = None,
+    use_cache: bool = True,
 ):
     if issubclass(cls, ShortFactBase):
         return get_short_facts(
@@ -139,6 +197,9 @@ def get_fact(
             args=args,
             kwargs=kwargs,
             ensure_hosts=ensure_hosts,
+            apply_failed_hosts=apply_failed_hosts,
+            fact_hash=fact_hash,
+            use_cache=use_cache,
         )
 
     with host.facts_lock:
@@ -158,48 +219,26 @@ def get_fact(
 
 
 def _get_fact(
-    state,
-    host,
-    cls,
-    args=None,
-    kwargs=None,
-    ensure_hosts=None,
-    apply_failed_hosts=True,
-    fact_hash=None,
+    state: "State",
+    host: "Host",
+    cls: Type[FactBase],
+    args: Optional[List] = None,
+    kwargs: Optional[Dict] = None,
+    ensure_hosts: Optional[Any] = None,
+    apply_failed_hosts: bool = True,
+    fact_hash: Optional[Any] = None,
 ):
     fact = cls()
     name = fact.name
 
-    args = args or ()
-    kwargs = kwargs or {}
+    fact_kwargs, executor_kwargs = _handle_fact_kwargs(state, host, cls, args, kwargs)
 
-    # Get the defaults *and* overrides by popping from kwargs, executor kwargs passed
-    # into get_fact override everything else (applied below).
-    override_kwargs, override_kwarg_keys = pop_global_arguments(
-        kwargs,
-        state=state,
-        host=host,
-        keys_to_check=get_executor_kwarg_keys(),
-    )
-
-    executor_kwargs = _get_executor_kwargs(
-        state,
-        host,
-        override_kwargs=override_kwargs,
-        override_kwarg_keys=override_kwarg_keys,
-    )
-
-    if args or kwargs:
-        # Merges args & kwargs into a single kwargs dictionary
-        kwargs = getcallargs(fact.command, *args, **kwargs)
-
-    kwargs_str = get_kwargs_str(kwargs)
+    kwargs_str = get_kwargs_str(fact_kwargs)
     logger.debug(
-        "Getting fact: {0} ({1}) (ensure_hosts: {2})".format(
-            name,
-            kwargs_str,
-            ensure_hosts,
-        ),
+        "Getting fact: %s (%s) (ensure_hosts: %r)",
+        name,
+        kwargs_str,
+        ensure_hosts,
     )
 
     if not host.connected:
@@ -217,8 +256,8 @@ def _get_fact(
     if fact.shell_executable:
         executor_kwargs["shell_executable"] = fact.shell_executable
 
-    command = _make_command(fact.command, kwargs)
-    requires_command = _make_command(fact.requires_command, kwargs)
+    command = _make_command(fact.command, fact_kwargs)
+    requires_command = _make_command(fact.requires_command, fact_kwargs)
     if requires_command:
         command = StringCommand(
             # Command doesn't exist, return 0 *or* run & return fact command
@@ -287,7 +326,7 @@ def _get_fact(
         log_error_or_warning(
             host,
             ignore_errors,
-            description=("could not load fact: {0} {1}").format(name, get_kwargs_str(kwargs)),
+            description=("could not load fact: {0} {1}").format(name, get_kwargs_str(fact_kwargs)),
         )
 
     # Check we've not failed
@@ -299,18 +338,31 @@ def _get_fact(
     return data
 
 
-def _get_fact_hash(state, host, cls, args, kwargs):
-    args = args or None
-    kwargs = kwargs or None
-    return make_hash((cls, args, kwargs, _get_executor_kwargs(state, host)))
+def _get_fact_hash(state: "State", host: "Host", cls, args, kwargs):
+    if issubclass(cls, ShortFactBase):
+        cls = cls.fact
+    fact_kwargs, executor_kwargs = _handle_fact_kwargs(state, host, cls, args, kwargs)
+    return make_hash((cls, fact_kwargs, executor_kwargs))
 
 
-def get_host_fact(state, host, cls, args=None, kwargs=None):
+def get_host_fact(
+    state: "State",
+    host: "Host",
+    cls,
+    args: Optional[List] = None,
+    kwargs: Optional[Dict] = None,
+):
     fact_hash = _get_fact_hash(state, host, cls, args, kwargs)
     return get_fact(state, host, cls, args=args, kwargs=kwargs, fact_hash=fact_hash)
 
 
-def reload_host_fact(state, host, cls, args=None, kwargs=None):
+def reload_host_fact(
+    state: "State",
+    host: "Host",
+    cls,
+    args: Optional[List] = None,
+    kwargs: Optional[Dict] = None,
+):
     fact_hash = _get_fact_hash(state, host, cls, args, kwargs)
     return get_fact(
         state,
@@ -323,11 +375,24 @@ def reload_host_fact(state, host, cls, args=None, kwargs=None):
     )
 
 
-def create_host_fact(state, host, cls, data, args=None, kwargs=None):
+def create_host_fact(
+    state: "State",
+    host: "Host",
+    cls,
+    data,
+    args: Optional[List] = None,
+    kwargs: Optional[Dict] = None,
+):
     fact_hash = _get_fact_hash(state, host, cls, args, kwargs)
     host.facts[fact_hash] = data
 
 
-def delete_host_fact(state, host, cls, args=None, kwargs=None):
+def delete_host_fact(
+    state: "State",
+    host: "Host",
+    cls,
+    args: Optional[List] = None,
+    kwargs: Optional[Dict] = None,
+):
     fact_hash = _get_fact_hash(state, host, cls, args, kwargs)
     host.facts.pop(fact_hash, None)
