@@ -1,4 +1,5 @@
 from contextlib import contextmanager
+from typing import TYPE_CHECKING, Any, Callable, Dict, Generator, List, Optional, Union
 
 import click
 from gevent.lock import BoundedSemaphore
@@ -10,8 +11,12 @@ from .connectors import get_execution_connector
 from .exceptions import ConnectError
 from .facts import create_host_fact, delete_host_fact, get_host_fact, reload_host_fact
 
+if TYPE_CHECKING:
+    from pyinfra.api.inventory import Inventory
+    from pyinfra.api.state import State
 
-def extract_callable_datas(datas):
+
+def extract_callable_datas(datas: List[Union[Callable[..., Any], Any]]) -> Generator[Any, Any, Any]:
     for data in datas:
         # Support for dynamic data, ie @deploy wrapped data defaults where
         # the data is stored on the state temporarily.
@@ -21,25 +26,25 @@ def extract_callable_datas(datas):
         yield data
 
 
-class HostData(object):
+class HostData:
     """
     Combines multiple AttrData's to search for attributes.
     """
 
-    override_datas = None
+    override_datas: Dict[str, Any]
 
-    def __init__(self, host, *datas):
+    def __init__(self, host: "Host", *datas):
         self.__dict__["host"] = host
 
-        datas = list(datas)
+        parsed_datas = list(datas)
 
         # Inject an empty override data so we can assign during deploy
         self.__dict__["override_datas"] = {}
-        datas.insert(0, self.override_datas)
+        parsed_datas.insert(0, self.override_datas)
 
-        self.__dict__["datas"] = tuple(datas)
+        self.__dict__["datas"] = tuple(parsed_datas)
 
-    def __getattr__(self, key):
+    def __getattr__(self, key: str):
         for data in extract_callable_datas(self.datas):
             try:
                 return data[key]
@@ -48,13 +53,13 @@ class HostData(object):
 
         raise AttributeError(f"Host `{self.host}` has no data `{key}`")
 
-    def __setattr__(self, key, value):
+    def __setattr__(self, key: str, value: Any):
         self.override_datas[key] = value
 
     def __str__(self):
         return str(self.datas)
 
-    def get(self, key, default=None):
+    def get(self, key: str, default=None):
         return getattr(self, key, default)
 
     def dict(self):
@@ -71,23 +76,23 @@ class HostData(object):
         return out
 
 
-class Host(object):
+class Host:
     """
     Represents a target host. Thin class that links up to facts and host/group
     data.
     """
 
     connection = None
-    state = None
+    state: "State"
 
     # Current context inside an @operation function (op gen stage)
-    in_op = False
-    current_op_hash = None
-    current_op_global_kwargs = None
+    in_op: bool = False
+    current_op_hash: Optional[str] = None
+    current_op_global_kwargs: Dict[str, Any]
 
     # Current context inside a @deploy function (op gen stage)
-    in_deploy = False
-    current_deploy_name = None
+    in_deploy: bool = False
+    current_deploy_name: Optional[str] = None
     current_deploy_kwargs = None
     current_deploy_data = None
 
@@ -95,10 +100,22 @@ class Host(object):
     executing_op_hash = None
     nested_executing_op_hash = None
 
+    loop_position: List[int]
+
+    # Arbitrary data dictionary connectors may use
+    connector_data: Dict[str, Any]
+
+    def loop(self, iterable):
+        self.loop_position.append(0)
+        for i, item in enumerate(iterable):
+            self.loop_position[-1] = i
+            yield item
+        self.loop_position.pop()
+
     def __init__(
         self,
-        name,
-        inventory,
+        name: str,
+        inventory: "Inventory",
         groups,
         executor=get_execution_connector("ssh"),
     ):
@@ -107,16 +124,19 @@ class Host(object):
         self.executor = executor
         self.name = name
 
-        # Arbitrary dict for connector use
+        self.loop_position = []
+
         self.connector_data = {}
+        self.current_op_global_kwargs = {}
 
         # Fact data store
-        self.facts = {}
+        # TODO: how to not have Any here?
+        self.facts: Dict[str, Any] = {}
         self.facts_lock = BoundedSemaphore()
 
         # Append only list of operation hashes as called on this host, used to
         # generate a DAG to create the final operation order.
-        self.op_hash_order = []
+        self.op_hash_order: List[str] = []
 
         # Create the (waterfall data: override, host, group, global)
         self.data = HostData(
@@ -136,7 +156,7 @@ class Host(object):
         return "Host({0})".format(self.name)
 
     @property
-    def connected(self):
+    def connected(self) -> bool:
         return self.connection is not None
 
     @property
@@ -182,7 +202,7 @@ class Host(object):
         handler("{0}noop: {1}".format(self.print_prefix, description))
 
     @contextmanager
-    def deploy(self, name, kwargs, data, in_deploy=True):
+    def deploy(self, name: str, kwargs, data, in_deploy: bool = True):
         """
         Wraps a group of operations as a deploy, this should not be used
         directly, instead use ``pyinfra.api.deploy.deploy``.
@@ -204,11 +224,10 @@ class Host(object):
         self.current_deploy_kwargs = kwargs
         self.current_deploy_data = data
         logger.debug(
-            "Starting deploy {0} (args={1}, data={2})".format(
-                name,
-                kwargs,
-                data,
-            ),
+            "Starting deploy %s (args=%r, data=%r)",
+            name,
+            kwargs,
+            data,
         )
 
         yield
@@ -220,26 +239,38 @@ class Host(object):
         self.current_deploy_data = old_deploy_data
 
         logger.debug(
-            "Reset deploy to {0} (args={1}, data={2})".format(
-                old_deploy_name,
-                old_deploy_kwargs,
-                old_deploy_data,
-            ),
+            "Reset deploy to %s (args=%r, data=%r)",
+            old_deploy_name,
+            old_deploy_kwargs,
+            old_deploy_data,
         )
 
     # Host facts
     #
 
     def get_fact(self, name_or_cls, *args, **kwargs):
+        """
+        Get a fact for this host, reading from the cache if present.
+        """
         return get_host_fact(self.state, self, name_or_cls, args=args, kwargs=kwargs)
 
     def reload_fact(self, name_or_cls, *args, **kwargs):
+        """
+        Get a fact for this host without using any cached value, always re-fetch the fact data
+        from the host and then cache it.
+        """
         return reload_host_fact(self.state, self, name_or_cls, args=args, kwargs=kwargs)
 
     def create_fact(self, name_or_cls, data=None, kwargs=None):
+        """
+        Create a new fact for this host in the fact cache.
+        """
         return create_host_fact(self.state, self, name_or_cls, data, kwargs=kwargs)
 
     def delete_fact(self, name_or_cls, kwargs=None):
+        """
+        Remove an existing fact for this host in the fact cache.
+        """
         return delete_host_fact(self.state, self, name_or_cls, kwargs=kwargs)
 
     # Connector proxy
@@ -249,7 +280,11 @@ class Host(object):
         if not self.state:
             raise TypeError("Cannot call this function with no state!")
 
-    def connect(self, reason=None, show_errors=True, raise_exceptions=False):
+    def connect(self, reason=None, show_errors: bool = True, raise_exceptions: bool = False):
+        """
+        Connect to the host using it's configured connector.
+        """
+
         self._check_state()
         if not self.connection:
             self.state.trigger_callbacks("host_before_connect", self)
@@ -285,6 +320,9 @@ class Host(object):
         return self.connection
 
     def disconnect(self):
+        """
+        Disconnect from the host using it's configured connector.
+        """
         self._check_state()
 
         # Disconnect is an optional function for executors if needed
@@ -298,14 +336,23 @@ class Host(object):
         self.state.trigger_callbacks("host_disconnect", self)
 
     def run_shell_command(self, *args, **kwargs):
+        """
+        Low level method to execute a shell command on the host via it's configured connector.
+        """
         self._check_state()
         return self.executor.run_shell_command(self.state, self, *args, **kwargs)
 
     def put_file(self, *args, **kwargs):
+        """
+        Low level method to upload a file to the host via it's configured connector.
+        """
         self._check_state()
         return self.executor.put_file(self.state, self, *args, **kwargs)
 
     def get_file(self, *args, **kwargs):
+        """
+        Low level method to download a file from the host via it's configured connector.
+        """
         self._check_state()
         return self.executor.get_file(self.state, self, *args, **kwargs)
 
