@@ -1,11 +1,15 @@
 from collections import defaultdict
+from typing import TYPE_CHECKING, Any, Iterator, List
 
 from .connectors import get_all_connectors, get_execution_connectors
 from .exceptions import NoConnectorError, NoGroupError, NoHostError
 from .host import Host
 
+if TYPE_CHECKING:
+    from pyinfra.api.state import State
 
-def extract_name_data(names):
+
+def extract_name_data(names: List[Any]):
     for name in names:
         data = {}
 
@@ -16,7 +20,7 @@ def extract_name_data(names):
         yield name, data
 
 
-class Inventory(object):
+class Inventory:
     """
     Represents a collection of target hosts. Stores and provides access to group data,
     host data and default data for these hosts.
@@ -29,7 +33,7 @@ class Inventory(object):
         **groups: map of group name -> ``(names, data)``
     """
 
-    state = None
+    state: "State"
 
     def __init__(self, names_data, override_data=None, **groups):
         # Setup basics
@@ -118,7 +122,7 @@ class Inventory(object):
                     )
 
         # Now we can actually make Host instances
-        hosts = {}
+        hosts: dict[str, "Host"] = {}
 
         for name, executor in names_executors:
             host_groups = name_to_group_names[name]
@@ -133,49 +137,45 @@ class Inventory(object):
 
         return hosts
 
-    def __len__(self):
+    def __len__(self) -> int:
         """
         Returns the number of inventory hosts.
         """
 
         return len(self.hosts)
 
-    def __iter__(self):
+    def __iter__(self) -> Iterator["Host"]:
         """
         Iterates over all inventory hosts.
         """
 
         return iter(self.hosts.values())
 
-    def iter_active_hosts(self):
+    def iter_active_hosts(self) -> Iterator["Host"]:
         """
         Iterates over active inventory hosts.
         """
-
         return iter(self.state.active_hosts)
 
-    def len_active_hosts(self):
+    def len_active_hosts(self) -> int:
         """
         Returns the number of active inventory hosts.
         """
-
         return len(self.state.active_hosts)
 
-    def iter_activated_hosts(self):
+    def iter_activated_hosts(self) -> Iterator["Host"]:
         """
         Iterates over activated inventory hosts.
         """
-
         return iter(self.state.activated_hosts)
 
-    def len_activated_hosts(self):
+    def len_activated_hosts(self) -> int:
         """
         Returns the number of activated inventory hosts.
         """
-
         return len(self.state.activated_hosts)
 
-    def get_host(self, name, default=NoHostError):
+    def get_host(self, name: str, default=NoHostError):
         """
         Get a single host by name.
         """
@@ -188,7 +188,7 @@ class Inventory(object):
 
         return default
 
-    def get_group(self, name, default=NoGroupError):
+    def get_group(self, name: str, default=NoGroupError):
         """
         Get a list of hosts belonging to a group.
         """
@@ -215,7 +215,7 @@ class Inventory(object):
 
         return self.override_data
 
-    def get_host_data(self, hostname):
+    def get_host_data(self, hostname: str):
         """
         Get data for a single host in this inventory.
         """
