@@ -1,6 +1,7 @@
 import traceback
 from itertools import product
 from socket import error as socket_error, timeout as timeout_error
+from typing import TYPE_CHECKING
 
 import click
 import gevent
@@ -13,7 +14,7 @@ from pyinfra.progress import progress_spinner
 
 from .arguments import get_executor_kwarg_keys
 from .command import FunctionCommand, PyinfraCommand, StringCommand
-from .exceptions import PyinfraError
+from .exceptions import NoMoreHostsError, PyinfraError
 from .util import (
     format_exception,
     log_error_or_warning,
@@ -23,42 +24,17 @@ from .util import (
     print_host_combined_output,
 )
 
+if TYPE_CHECKING:
+    from .inventory import Host
+    from .state import State
+
 
 @memoize
 def show_pre_or_post_condition_warning(condition_name):
     logger.warning("The `{0}` argument is in beta!".format(condition_name))
 
 
-def _run_shell_command(
-    state,
-    host,
-    command,
-    global_kwargs,
-    executor_kwargs,
-    return_combined_output=False,
-):
-    status = False
-    combined_output_lines = []
-
-    try:
-        status, combined_output_lines = command.execute(state, host, executor_kwargs)
-    except (timeout_error, socket_error, SSHException) as e:
-        log_host_command_error(
-            host,
-            e,
-            timeout=global_kwargs["timeout"],
-        )
-
-    # If we failed and have no already printed the stderr, print it
-    if status is False and not state.print_output:
-        print_host_combined_output(host, combined_output_lines)
-
-    if return_combined_output:
-        return status, combined_output_lines
-    return status
-
-
-def run_host_op(state, host, op_hash):
+def run_host_op(state: "State", host: "Host", op_hash):
     state.trigger_callbacks("operation_host_start", host, op_hash)
 
     if op_hash not in state.ops[host]:
@@ -73,36 +49,58 @@ def run_host_op(state, host, op_hash):
     ignore_errors = global_kwargs["ignore_errors"]
     continue_on_error = global_kwargs["continue_on_error"]
 
-    logger.debug(
-        "Starting operation {0} on {1}".format(
-            ", ".join(op_meta["names"]),
-            host,
-        ),
-    )
+    logger.debug("Starting operation %r on %s", op_meta["names"], host)
 
     executor_kwarg_keys = get_executor_kwarg_keys()
     base_executor_kwargs = {
         key: global_kwargs[key] for key in executor_kwarg_keys if key in global_kwargs
     }
 
-    precondition = global_kwargs["precondition"]
-    if precondition:
-        show_pre_or_post_condition_warning("precondition")
-    if precondition and not _run_shell_command(
-        state,
-        host,
-        StringCommand(precondition),
-        global_kwargs,
-        base_executor_kwargs,
-    ):
-        log_error_or_warning(
-            host,
-            ignore_errors,
-            description="precondition failed: {0}".format(precondition),
+    def _run_shell_command(command, executor_kwargs):
+        status = False
+        combined_output_lines = []
+
+        try:
+            status, combined_output_lines = command.execute(state, host, executor_kwargs)
+        except (timeout_error, socket_error, SSHException) as e:
+            log_host_command_error(
+                host,
+                e,
+                timeout=global_kwargs["timeout"],
+            )
+
+        # If we failed and have no already printed the stderr, print it
+        if status is False and not state.print_output:
+            print_host_combined_output(host, combined_output_lines)
+
+        return status, combined_output_lines
+
+    def run_condition(condition_name: str) -> bool:
+        condition_value = global_kwargs[condition_name]
+        if not condition_value:
+            return True
+
+        show_pre_or_post_condition_warning(condition_name)
+
+        _shell_command_status, _ = _run_shell_command(
+            StringCommand(condition_value),
+            base_executor_kwargs,
         )
-        if not ignore_errors:
-            state.trigger_callbacks("operation_host_error", host, op_hash)
-            return False
+
+        if _shell_command_status:
+            return True
+
+        _log_msg = f"{condition_name} failed: {condition_value}"
+        log_error_or_warning(host, ignore_errors, description=_log_msg)
+
+        if ignore_errors:
+            return True
+
+        state.trigger_callbacks("operation_host_error", host, op_hash)
+        return False
+
+    if not run_condition("precondition"):
+        return False
 
     state.ops_run.add(op_hash)
 
@@ -117,7 +115,6 @@ def run_host_op(state, host, op_hash):
     all_combined_output_lines = []
 
     for i, command in enumerate(op_data["commands"]):
-
         status = False
 
         executor_kwargs = base_executor_kwargs.copy()
@@ -132,40 +129,26 @@ def run_host_op(state, host, op_hash):
         if isinstance(command, FunctionCommand):
             try:
                 status = command.execute(state, host, executor_kwargs)
+            except NoMoreHostsError:
+                status = False
             except Exception as e:  # Custom functions could do anything, so expect anything!
+                _formatted_exc = format_exception(e)
+                _error_msg = "Unexpected error in Python callback: {0}".format(_formatted_exc)
+                _error_msg_styled = click.style(_error_msg, "red")
+                _error_log = "{0}{1}".format(host.print_prefix, _error_msg_styled)
                 logger.warning(traceback.format_exc())
-                logger.error(
-                    "{0}{1}".format(
-                        host.print_prefix,
-                        click.style(
-                            "Unexpected error in Python callback: {0}".format(
-                                format_exception(e),
-                            ),
-                            "red",
-                        ),
-                    ),
-                )
+                logger.error(_error_log)
 
         elif isinstance(command, StringCommand):
-            status, combined_output_lines = _run_shell_command(
-                state,
-                host,
-                command,
-                global_kwargs,
-                executor_kwargs,
-                return_combined_output=True,
-            )
+            status, combined_output_lines = _run_shell_command(command, executor_kwargs)
             all_combined_output_lines.extend(combined_output_lines)
 
         else:
             try:
                 status = command.execute(state, host, executor_kwargs)
             except (timeout_error, socket_error, SSHException, IOError) as e:
-                log_host_command_error(
-                    host,
-                    e,
-                    timeout=global_kwargs["timeout"],
-                )
+                _timeout = global_kwargs["timeout"]
+                log_host_command_error(host, e, timeout=_timeout)
 
         # Break the loop to trigger a failure
         if status is False:
@@ -179,24 +162,8 @@ def run_host_op(state, host, op_hash):
 
     # Commands didn't break, so count our successes & return True!
     else:
-        postcondition = global_kwargs["postcondition"]
-        if postcondition:
-            show_pre_or_post_condition_warning("postcondition")
-        if postcondition and not _run_shell_command(
-            state,
-            host,
-            StringCommand(postcondition),
-            global_kwargs,
-            base_executor_kwargs,
-        ):
-            log_error_or_warning(
-                host,
-                ignore_errors,
-                description="postcondition failed: {0}".format(postcondition),
-            )
-            if not ignore_errors:
-                state.trigger_callbacks("operation_host_error", host, op_hash)
-                return False
+        if not run_condition("postcondition"):
+            return False
 
         if not did_error:
             return_status = True
@@ -205,15 +172,9 @@ def run_host_op(state, host, op_hash):
         state.results[host]["ops"] += 1
         state.results[host]["success_ops"] += 1
 
-        logger.info(
-            "{0}{1}".format(
-                host.print_prefix,
-                click.style(
-                    "Success" if len(op_data["commands"]) > 0 else "No changes",
-                    "green",
-                ),
-            ),
-        )
+        _status_log = "Success" if len(op_data["commands"]) > 0 else "No changes"
+        _click_log_status = click.style(_status_log, "green")
+        logger.info("{0}{1}".format(host.print_prefix, _click_log_status))
 
         # Trigger any success handler
         if global_kwargs["on_success"]:
@@ -229,12 +190,8 @@ def run_host_op(state, host, op_hash):
         if executed_commands:
             state.results[host]["partial_ops"] += 1
 
-        log_error_or_warning(
-            host,
-            ignore_errors,
-            continue_on_error=continue_on_error,
-            description=f"executed {executed_commands}/{len(op_data['commands'])} commands",
-        )
+        _command_description = f"executed {executed_commands}/{len(op_data['commands'])} commands"
+        log_error_or_warning(host, ignore_errors, _command_description, continue_on_error)
 
         # Always trigger any error handler
         if global_kwargs["on_error"]:
@@ -260,17 +217,17 @@ def run_host_op(state, host, op_hash):
     return return_status
 
 
-def _run_host_op_with_context(state, host, op_hash):
+def _run_host_op_with_context(state: "State", host: "Host", op_hash: str):
     with ctx_host.use(host):
         return run_host_op(state, host, op_hash)
 
 
-def _run_host_ops(state, host, progress=None):
+def _run_host_ops(state: "State", host: "Host", progress=None):
     """
     Run all ops for a single server.
     """
 
-    logger.debug("Running all ops on {0}".format(host))
+    logger.debug("Running all ops on %s", host)
 
     for op_hash in state.get_op_order():
         op_meta = state.get_op_meta(op_hash)
@@ -294,7 +251,7 @@ def _run_host_ops(state, host, progress=None):
             click.echo(err=True)
 
 
-def _run_serial_ops(state):
+def _run_serial_ops(state: "State"):
     """
     Run all ops for all servers, one server at a time.
     """
@@ -312,7 +269,7 @@ def _run_serial_ops(state):
                 state.fail_hosts({host})
 
 
-def _run_no_wait_ops(state):
+def _run_no_wait_ops(state: "State"):
     """
     Run all ops for all servers at once.
     """
@@ -320,6 +277,8 @@ def _run_no_wait_ops(state):
     hosts_operations = product(state.inventory.iter_active_hosts(), state.get_op_order())
     with progress_spinner(hosts_operations) as progress:
         # Spawn greenlet for each host to run *all* ops
+        if state.pool is None:
+            raise PyinfraError("No pool found on state.")
         greenlets = [
             state.pool.spawn(
                 _run_host_ops,
@@ -332,7 +291,7 @@ def _run_no_wait_ops(state):
         gevent.joinall(greenlets)
 
 
-def _run_single_op(state, op_hash):
+def _run_single_op(state: "State", op_hash: str):
     """
     Run a single operation for all servers. Can be configured to run in serial.
     """
@@ -368,6 +327,8 @@ def _run_single_op(state, op_hash):
         for batch in batches:
             with progress_spinner(batch) as progress:
                 # Spawn greenlet for each host
+                if state.pool is None:
+                    raise PyinfraError("No pool found on state.")
                 greenlet_to_host = {
                     state.pool.spawn(_run_host_op_with_context, state, host, op_hash): host
                     for host in batch
@@ -392,7 +353,7 @@ def _run_single_op(state, op_hash):
     state.trigger_callbacks("operation_end", op_hash)
 
 
-def run_ops(state, serial=False, no_wait=False):
+def run_ops(state: "State", serial: bool = False, no_wait: bool = False):
     """
     Runs all operations across all servers in a configurable manner.
 
