@@ -12,29 +12,32 @@ The `@ansible` connector can be used to parse Ansible inventory files.
     # Load using an absolute path
     pyinfra @ansible//absolute/path/to/inventory
 """
-
 import json
 import re
 from collections import defaultdict
 from configparser import ConfigParser
 from os import path
-
-try:
-    import yaml
-except ImportError:
-    yaml = None
+from typing import TYPE_CHECKING, Optional
 
 from pyinfra import logger
 from pyinfra.api.exceptions import InventoryError
 from pyinfra.api.util import memoize
 
+if TYPE_CHECKING:
+    from pyinfra.api.host import Host
+
+try:
+    import yaml
+except ImportError:
+    yaml = None  # type: ignore
+
 
 @memoize
 def show_warning():
     logger.warning("The @ansible connector is in alpha!")
 
 
-def make_names_data(inventory_filename=None):
+def make_names_data(inventory_filename: Optional[str] = None):
     show_warning()
 
     if not inventory_filename:
@@ -48,7 +51,7 @@ def make_names_data(inventory_filename=None):
     return parse_inventory(inventory_filename)
 
 
-def parse_inventory(inventory_filename):
+def parse_inventory(inventory_filename: str):
     # fallback to INI if no extension
     extension = inventory_filename.split(".")[-1] if "." in inventory_filename else "ini"
 
@@ -58,7 +61,7 @@ def parse_inventory(inventory_filename):
     if extension in ["ini"]:
         host_to_groups = parse_inventory_ini(inventory_filename)
     elif extension in ["json"]:
-        with open(inventory_filename) as inventory_file:
+        with open(inventory_filename, encoding="utf-8") as inventory_file:
             inventory_tree = json.load(inventory_file)
             # close file early
         host_to_groups = parse_inventory_tree(inventory_tree)
@@ -70,17 +73,17 @@ def parse_inventory(inventory_filename):
                     "Install it with `pip install pyyaml`."
                 ),
             )
-        with open(inventory_filename) as inventory_file:
+        with open(inventory_filename, encoding="utf-8") as inventory_file:
             inventory_tree = yaml.safe_load(inventory_file)
             # close file early
         host_to_groups = parse_inventory_tree(inventory_tree)
     else:
         raise InventoryError(("Ansible inventory file format not supported: {0}").format(extension))
 
-    return [(host, {}, sorted(list(host_to_groups.get(host)))) for host in host_to_groups]
+    return [(host, {}, sorted(list(host_to_groups.get(host, [])))) for host in host_to_groups]
 
 
-def parse_inventory_ini(inventory_filename):
+def parse_inventory_ini(inventory_filename: str):
     config = ConfigParser(
         delimiters=(" "),  # we only handle the hostnames for now
         allow_no_value=True,  # we don't by default have = values
@@ -160,7 +163,7 @@ def parse_inventory_tree(inventory_tree, host_to_groups=dict(), group_stack=set(
     return host_to_groups
 
 
-def append_groups_to_host(host, groups, host_to_groups):
+def append_groups_to_host(host: "Host", groups, host_to_groups):
     if host in host_to_groups:
         # set logic handles de-duplication
         host_to_groups[host] = host_to_groups[host].union(groups)
