@@ -5,10 +5,12 @@ The ``@local`` connector executes changes on the local machine using subprocesse
 import os
 from distutils.spawn import find_executable
 from tempfile import mkstemp
+from typing import TYPE_CHECKING
 
 import click
 
 from pyinfra import logger
+from pyinfra.api.command import QuoteString, StringCommand
 from pyinfra.api.connectors import BaseConnectorMeta
 from pyinfra.api.exceptions import InventoryError
 from pyinfra.api.util import get_file_io
@@ -20,6 +22,10 @@ from .util import (
     split_combined_output,
 )
 
+if TYPE_CHECKING:
+    from pyinfra.api.host import Host
+    from pyinfra.api.state import State
+
 
 class Meta(BaseConnectorMeta):
     handles_execution = True
@@ -32,21 +38,21 @@ def make_names_data(_=None):
     yield "@local", {}, ["@local"]
 
 
-def connect(state, host):
+def connect(state: "State", host: "Host"):
     return True
 
 
 def run_shell_command(
-    state,
-    host,
+    state: "State",
+    host: "Host",
     command,
-    get_pty=False,  # ignored
+    get_pty: bool = False,  # ignored
     timeout=None,
     stdin=None,
     success_exit_codes=None,
-    print_output=False,
-    print_input=False,
-    return_combined_output=False,
+    print_output: bool = False,
+    print_input: bool = False,
+    return_combined_output: bool = False,
     **command_kwargs,
 ):
     """
@@ -70,7 +76,7 @@ def run_shell_command(
         unix_command = make_unix_command_for_host(state, host, command, **command_kwargs)
         actual_command = unix_command.get_raw_value()
 
-        logger.debug("--> Running command on localhost: {0}".format(unix_command))
+        logger.debug("--> Running command on localhost: %s", unix_command)
 
         if print_input:
             click.echo("{0}>>> {1}".format(host.print_prefix, unix_command), err=True)
@@ -102,13 +108,13 @@ def run_shell_command(
 
 
 def put_file(
-    state,
-    host,
+    state: "State",
+    host: "Host",
     filename_or_io,
     remote_filename,
     remote_temp_filename=None,  # ignored
-    print_output=False,
-    print_input=False,
+    print_output: bool = False,
+    print_input: bool = False,
     **command_kwargs,
 ):
     """
@@ -133,7 +139,7 @@ def put_file(
         status, _, stderr = run_shell_command(
             state,
             host,
-            "cp {0} {1}".format(temp_filename, remote_filename),
+            StringCommand("cp", temp_filename, QuoteString(remote_filename)),
             print_output=print_output,
             print_input=print_input,
             **command_kwargs,
@@ -154,13 +160,13 @@ def put_file(
 
 
 def get_file(
-    state,
-    host,
+    state: "State",
+    host: "Host",
     remote_filename,
     filename_or_io,
     remote_temp_filename=None,  # ignored
-    print_output=False,
-    print_input=False,
+    print_output: bool = False,
+    print_input: bool = False,
     **command_kwargs,
 ):
     """
@@ -185,14 +191,17 @@ def get_file(
             raise IOError("\n".join(stderr))
 
         # Load our file or IO object and write it to the temporary file
-        with open(temp_filename) as temp_f:
+        with open(temp_filename, encoding="utf-8") as temp_f:
             with get_file_io(filename_or_io, "wb") as file_io:
+                data_bytes: bytes
+
                 data = temp_f.read()
-
                 if isinstance(data, str):
-                    data = data.encode()
+                    data_bytes = data.encode()
+                else:
+                    data_bytes = data
 
-                file_io.write(data)
+                file_io.write(data_bytes)
     finally:
         os.remove(temp_filename)
 
@@ -210,7 +219,16 @@ def check_can_rsync(host):
         raise NotImplementedError("The `rsync` binary is not available on this system.")
 
 
-def rsync(state, host, src, dest, flags, print_output=False, print_input=False, **command_kwargs):
+def rsync(
+    state: "State",
+    host: "Host",
+    src,
+    dest,
+    flags,
+    print_output: bool = False,
+    print_input: bool = False,
+    **command_kwargs,
+):
     status, _, stderr = run_shell_command(
         state,
         host,
