@@ -8,11 +8,14 @@ are identical:
     pyinfra my-host.net ...
     pyinfra @ssh/my-host.net ...
 """
-
+import shlex
 from distutils.spawn import find_executable
 from getpass import getpass
 from os import path
+from random import uniform
 from socket import error as socket_error, gaierror
+from time import sleep
+from typing import TYPE_CHECKING, Type, Union
 
 import click
 from paramiko import (
@@ -44,6 +47,10 @@ from .util import (
     write_stdin,
 )
 
+if TYPE_CHECKING:
+    from pyinfra.api.host import Host
+    from pyinfra.api.state import State
+
 
 class Meta(BaseConnectorMeta):
     handles_execution = True
@@ -66,6 +73,10 @@ class Meta(BaseConnectorMeta):
         known_hosts_file = "Custom SSH known hosts file"
         strict_host_key_checking = "Override strict host keys check setting"
 
+        connect_retries = "Number of tries to connect via ssh"
+        connect_retry_min_delay = "Lower bound for random delay between retries"
+        connect_retry_max_delay = "Upper bound for random delay between retries"
+
         paramiko_connect_kwargs = (
             "Override keyword arguments passed into paramiko's `SSHClient.connect`"
         )
@@ -78,13 +89,15 @@ def make_names_data(hostname):
     yield "@ssh/{0}".format(hostname), {DATA_KEYS.hostname: hostname}, []
 
 
-def _raise_connect_error(host, message, data):
+def _raise_connect_error(host: "Host", message, data):
     message = "{0} ({1})".format(message, data)
     raise ConnectError(message)
 
 
-def _load_private_key_file(filename, key_filename, key_password):
-    exception = PyinfraError("Invalid key: {0}".format(filename))
+def _load_private_key_file(filename: str, key_filename: str, key_password: str):
+    exception: Union[PyinfraError, SSHException] = PyinfraError("Invalid key: {0}".format(filename))
+
+    key_cls: Union[Type[RSAKey], Type[DSSKey], Type[ECDSAKey], Type[Ed25519Key]]
 
     for key_cls in (RSAKey, DSSKey, ECDSAKey, Ed25519Key):
         try:
@@ -123,7 +136,7 @@ def _load_private_key_file(filename, key_filename, key_password):
     raise exception
 
 
-def _get_private_key(state, key_filename, key_password):
+def _get_private_key(state: "State", key_filename: str, key_password: str):
     if key_filename in state.private_keys:
         return state.private_keys[key_filename]
 
@@ -136,7 +149,7 @@ def _get_private_key(state, key_filename, key_password):
         # Relative to the CWD
         path.join(state.cwd, key_filename)
 
-    key = False
+    key = None
     key_file_exists = False
 
     for filename in ssh_key_filenames:
@@ -170,7 +183,7 @@ def _get_private_key(state, key_filename, key_password):
     return key
 
 
-def _make_paramiko_kwargs(state, host):
+def _make_paramiko_kwargs(state: "State", host: "Host"):
     kwargs = {
         "allow_agent": False,
         "look_for_keys": False,
@@ -214,14 +227,22 @@ def _make_paramiko_kwargs(state, host):
     return kwargs
 
 
-def connect(state, host):
+def connect(state: "State", host: "Host"):
     """
-    Connect to a single host. Returns the SSH client if succesful. Stateless by
+    Connect to a single host. Returns the SSH client if successful. Stateless by
     design so can be run in parallel.
     """
+    retries = host.data.get(DATA_KEYS.connect_retries, 0)
 
+    for tries_left in range(retries, -1, -1):
+        con = _connect(state, host, tries_left)
+        if con:
+            return con
+
+
+def _connect(state: "State", host: "Host", tries_left: int):
     kwargs = _make_paramiko_kwargs(state, host)
-    logger.debug("Connecting to: {0} ({1})".format(host.name, kwargs))
+    logger.debug("Connecting to: %s (%r)", host.name, kwargs)
 
     hostname = kwargs.pop("hostname")
 
@@ -266,29 +287,39 @@ def connect(state, host):
         )
 
     except SSHException as e:
-        _raise_connect_error(host, "SSH error", e)
+        if tries_left == 0:
+            _raise_connect_error(host, "SSH error", e)
 
     except gaierror:
-        _raise_connect_error(host, "Could not resolve hostname", hostname)
+        if tries_left == 0:
+            _raise_connect_error(host, "Could not resolve hostname", hostname)
 
     except socket_error as e:
-        _raise_connect_error(host, "Could not connect", e)
+        if tries_left == 0:
+            _raise_connect_error(host, "Could not connect", e)
 
     except EOFError as e:
-        _raise_connect_error(host, "EOF error", e)
+        if tries_left == 0:
+            _raise_connect_error(host, "EOF error", e)
+
+    min_delay = host.data.get(DATA_KEYS.connect_retry_min_delay, 0.1)
+    max_delay = host.data.get(DATA_KEYS.connect_retry_max_delay, 0.5)
+    sleep(uniform(min_delay, max_delay))
+
+    return None
 
 
 def run_shell_command(
-    state,
-    host,
+    state: "State",
+    host: "Host",
     command,
-    get_pty=False,
+    get_pty: bool = False,
     timeout=None,
     stdin=None,
     success_exit_codes=None,
-    print_output=False,
-    print_input=False,
-    return_combined_output=False,
+    print_output: bool = False,
+    print_input: bool = False,
+    return_combined_output: bool = False,
     **command_kwargs,
 ):
     """
@@ -314,11 +345,10 @@ def run_shell_command(
         actual_command = unix_command.get_raw_value()
 
         logger.debug(
-            "Running command on {0}: (pty={1}) {2}".format(
-                host.name,
-                get_pty,
-                unix_command,
-            ),
+            "Running command on %s: (pty=%s) %s",
+            host.name,
+            get_pty,
+            unix_command,
         )
 
         if print_input:
@@ -343,7 +373,7 @@ def run_shell_command(
 
         logger.debug("Waiting for exit status...")
         exit_status = stdout_buffer.channel.recv_exit_status()
-        logger.debug("Command exit status: {0}".format(exit_status))
+        logger.debug("Command exit status: %i", exit_status)
 
         return exit_status, combined_output
 
@@ -366,7 +396,8 @@ def run_shell_command(
 
 
 @memoize
-def _get_sftp_connection(host):
+def _get_sftp_connection(host: "Host"):
+    assert host.connection is not None
     transport = host.connection.get_transport()
 
     try:
@@ -380,23 +411,23 @@ def _get_sftp_connection(host):
         ) from e
 
 
-def _get_file(host, remote_filename, filename_or_io):
+def _get_file(host: "Host", remote_filename: str, filename_or_io):
     with get_file_io(filename_or_io, "wb") as file_io:
         sftp = _get_sftp_connection(host)
         sftp.getfo(remote_filename, file_io)
 
 
 def get_file(
-    state,
-    host,
-    remote_filename,
+    state: "State",
+    host: "Host",
+    remote_filename: str,
     filename_or_io,
     remote_temp_filename=None,
-    sudo=False,
+    sudo: bool = False,
     sudo_user=None,
     su_user=None,
-    print_output=False,
-    print_input=False,
+    print_output: bool = False,
+    print_input: bool = False,
     **command_kwargs,
 ):
     """
@@ -462,31 +493,40 @@ def get_file(
     return True
 
 
-def _put_file(host, filename_or_io, remote_location):
-    attempts = 1
+def _put_file(host: "Host", filename_or_io, remote_location):
+    logger.debug("Attempting upload of %s to %s", filename_or_io, remote_location)
 
-    try:
-        with get_file_io(filename_or_io) as file_io:
-            sftp = _get_sftp_connection(host)
-            sftp.putfo(file_io, remote_location)
-    except OSError as e:
-        if attempts > 3:
-            raise
-        logger.warning(f"Failed to upload file, retrying: {e}")
-        attempts += 1
+    attempts = 0
+    last_e = None
+
+    while attempts < 3:
+        try:
+            with get_file_io(filename_or_io) as file_io:
+                sftp = _get_sftp_connection(host)
+                sftp.putfo(file_io, remote_location)
+            return
+        except OSError as e:
+            logger.warning(f"Failed to upload file, retrying: {e}")
+            attempts += 1
+            last_e = e
+
+    if last_e is not None:
+        raise last_e
 
 
 def put_file(
-    state,
-    host,
+    state: "State",
+    host: "Host",
     filename_or_io,
     remote_filename,
     remote_temp_filename=None,
-    sudo=False,
+    sudo: bool = False,
     sudo_user=None,
+    doas: bool = False,
+    doas_user=None,
     su_user=None,
-    print_output=False,
-    print_input=False,
+    print_output: bool = False,
+    print_input: bool = False,
     **command_kwargs,
 ):
     """
@@ -496,7 +536,7 @@ def put_file(
 
     # sudo/su are a little more complicated, as you can only sftp with the SSH
     # user connected, so upload to tmp and copy/chown w/sudo and/or su_user
-    if sudo or su_user:
+    if sudo or doas or su_user:
         # Get temp file location
         temp_file = remote_temp_filename or state.get_temp_filename(remote_filename)
         _put_file(host, filename_or_io, temp_file)
@@ -505,8 +545,11 @@ def put_file(
         if su_user:
             command = StringCommand("setfacl", "-m", "u:{0}:r".format(su_user), temp_file)
         elif sudo_user:
-            command = StringCommand("setfacl -m u:{0}:r".format(sudo_user), temp_file)
-        if su_user or sudo_user:
+            command = StringCommand("setfacl", "-m", "u:{0}:r".format(sudo_user), temp_file)
+        elif doas_user:
+            command = StringCommand("setfacl", "-m", "u:{0}:r".format(doas_user), temp_file)
+
+        if su_user or sudo_user or doas_user:
             status, _, stderr = run_shell_command(
                 state,
                 host,
@@ -530,6 +573,8 @@ def put_file(
             command,
             sudo=sudo,
             sudo_user=sudo_user,
+            doas=doas,
+            doas_user=doas_user,
             su_user=su_user,
             print_output=print_output,
             print_input=print_input,
@@ -548,6 +593,7 @@ def put_file(
             host,
             command,
             sudo=False,
+            doas=False,
             print_output=print_output,
             print_input=print_input,
             **command_kwargs,
@@ -570,7 +616,7 @@ def put_file(
     return True
 
 
-def check_can_rsync(host):
+def check_can_rsync(host: "Host"):
     if host.data.get(DATA_KEYS.key_password):
         raise NotImplementedError("Rsync does not currently work with SSH keys needing passwords.")
 
@@ -582,14 +628,14 @@ def check_can_rsync(host):
 
 
 def rsync(
-    state,
-    host,
-    src,
-    dest,
+    state: "State",
+    host: "Host",
+    src: str,
+    dest: str,
     flags,
-    print_output=False,
-    print_input=False,
-    sudo=False,
+    print_output: bool = False,
+    print_input: bool = False,
+    sudo: bool = False,
     sudo_user=None,
     **ignored_kwargs,
 ):
@@ -599,6 +645,24 @@ def rsync(
         user = "{0}@".format(user)
 
     ssh_flags = []
+    # To avoid asking for interactive input, specify BatchMode=yes
+    ssh_flags.append("-o BatchMode=yes")
+
+    known_hosts_file = host.data.get(DATA_KEYS.known_hosts_file, "")
+    if known_hosts_file:
+        ssh_flags.append(
+            '-o \\"UserKnownHostsFile={0}\\"'.format(shlex.quote(known_hosts_file))
+        )  # never trust users
+
+    strict_host_key_checking = host.data.get(DATA_KEYS.strict_host_key_checking, "")
+    if strict_host_key_checking:
+        ssh_flags.append(
+            '-o \\"StrictHostKeyChecking={0}\\"'.format(shlex.quote(strict_host_key_checking))
+        )
+
+    ssh_config_file = host.data.get(DATA_KEYS.config_file, "")
+    if ssh_config_file:
+        ssh_flags.append("-F {0}".format(shlex.quote(ssh_config_file)))
 
     port = host.data.get(DATA_KEYS.port)
     if port:
@@ -614,10 +678,9 @@ def rsync(
         if sudo_user:
             remote_rsync_command = "sudo -u {0} rsync".format(sudo_user)
 
-    # To avoid asking for interactive input, specify BatchMode=yes
     rsync_command = (
         "rsync {rsync_flags} "
-        "--rsh 'ssh -o BatchMode=yes {ssh_flags}' "
+        '--rsh "ssh {ssh_flags}" '
         "--rsync-path '{remote_rsync_command}' "
         "{src} {user}{hostname}:{dest}"
     ).format(
