@@ -3,7 +3,9 @@ The files facts provide information about the filesystem and it's contents on th
 """
 
 import re
+import stat
 from datetime import datetime
+from typing import List, Tuple
 
 from pyinfra.api.command import QuoteString, make_formatted_string_command
 from pyinfra.api.facts import FactBase
@@ -28,31 +30,38 @@ FLAG_TO_TYPE = {
     "-": "file",
 }
 
-SYMBOL_TO_OCTAL_PERMISSIONS = {
-    "rwx": "7",
-    "rw-": "6",
-    "r-x": "5",
-    "r--": "4",
-    "-wx": "3",
-    "-w-": "2",
-    "--x": "1",
-}
+# Each item is a map of character to permission octal to be combined, taken from stdlib:
+# https://github.com/python/cpython/blob/c1c3be0f9dc414bfae9a5718451ca217751ac687/Lib/stat.py#L128-L154
+CHAR_TO_PERMISSION = (
+    # User
+    {"r": stat.S_IRUSR},
+    {"w": stat.S_IWUSR},
+    {"x": stat.S_IXUSR, "S": stat.S_ISUID, "s": stat.S_IXUSR | stat.S_ISUID},
+    # Group
+    {"r": stat.S_IRGRP},
+    {"w": stat.S_IWGRP},
+    {"x": stat.S_IXGRP, "S": stat.S_ISGID, "s": stat.S_IXGRP | stat.S_ISGID},
+    # Other
+    {"r": stat.S_IROTH},
+    {"w": stat.S_IWOTH},
+    {"x": stat.S_IXOTH, "T": stat.S_ISVTX, "t": stat.S_IXOTH | stat.S_ISVTX},
+)
 
 
-def _parse_mode(mode):
+def _parse_mode(mode: str) -> int:
     """
-    Converts ls mode output (rwxrwxrwx) -> integer (755).
+    Converts ls mode output (rwxrwxrwx) -> octal permission integer (755).
     """
 
-    result = ""
-    # owner, group, world
-    for group in [mode[0:3], mode[3:6], mode[6:9]]:
-        if group in SYMBOL_TO_OCTAL_PERMISSIONS:
-            result = "{0}{1}".format(result, SYMBOL_TO_OCTAL_PERMISSIONS[group])
-        else:
-            result = "{0}0".format(result)
+    out = 0
 
-    return int(result)
+    for i, char in enumerate(mode):
+        for c, m in CHAR_TO_PERMISSION[i].items():
+            if char == c:
+                out |= m
+                break
+
+    return int(oct(out)[2:])
 
 
 class File(FactBase):
@@ -196,91 +205,57 @@ class Socket(File):
     type = "socket"
 
 
-class Sha1File(FactBase):
+class HashFileFactBase(FactBase):
+    _raw_cmd: str
+    _regexes: Tuple[str, str]
+
+    def __init_subclass__(cls, digits: int, cmds: List[str], **kwargs) -> None:
+        super().__init_subclass__(**kwargs)
+
+        raw_hash_cmds = ["%s {0} 2> /dev/null" % cmd for cmd in cmds]
+        raw_hash_cmd = " || ".join(raw_hash_cmds)
+        cls._raw_cmd = "test -e {0} && ( %s ) || true" % raw_hash_cmd
+
+        assert cls.__name__.endswith("File")
+        hash_name = cls.__name__[:-4].upper()
+        cls._regexes = (
+            # GNU coreutils style:
+            r"^([a-fA-F0-9]{%d})\s+%%s$" % digits,
+            # BSD style:
+            r"^%s\s+\(%%s\)\s+=\s+([a-fA-F0-9]{%d})$" % (hash_name, digits),
+        )
+
+    def command(self, path):
+        self.path = path
+        return make_formatted_string_command(self._raw_cmd, QuoteString(path))
+
+    def process(self, output):
+        output = output[0]
+        escaped_path = re.escape(self.path)
+        for regex in self._regexes:
+            matches = re.match(regex % escaped_path, output)
+            if matches:
+                return matches.group(1)
+
+
+class Sha1File(HashFileFactBase, digits=40, cmds=["sha1sum", "shasum", "sha1"]):
     """
     Returns a SHA1 hash of a file. Works with both sha1sum and sha1. Returns
     ``None`` if the file doest not exist.
     """
 
-    _regexes = [
-        r"^([a-zA-Z0-9]{40})\s+%s$",
-        r"^SHA1\s+\(%s\)\s+=\s+([a-zA-Z0-9]{40})$",
-    ]
 
-    def command(self, path):
-        self.path = path
-        return make_formatted_string_command(
-            (
-                "test -e {0} && ( "
-                "sha1sum {0} 2> /dev/null || shasum {0} 2> /dev/null || sha1 {0} "
-                ") || true"
-            ),
-            QuoteString(path),
-        )
-
-    def process(self, output):
-        for regex in self._regexes:
-            regex = regex % re.escape(self.path)
-            matches = re.match(regex, output[0])
-            if matches:
-                return matches.group(1)
-
-
-class Sha256File(FactBase):
+class Sha256File(HashFileFactBase, digits=64, cmds=["sha256sum", "shasum -a 256", "sha256"]):
     """
     Returns a SHA256 hash of a file, or ``None`` if the file does not exist.
     """
 
-    _regexes = [
-        r"^([a-zA-Z0-9]{64})\s+%s$",
-        r"^SHA256\s+\(%s\)\s+=\s+([a-zA-Z0-9]{64})$",
-    ]
 
-    def command(self, path):
-        self.path = path
-        return make_formatted_string_command(
-            (
-                "test -e {0} && ( "
-                "sha256sum {0} 2> /dev/null "
-                "|| shasum -a 256 {0} 2> /dev/null "
-                "|| sha256 {0} "
-                ") || true"
-            ),
-            QuoteString(path),
-        )
-
-    def process(self, output):
-        for regex in self._regexes:
-            regex = regex % re.escape(self.path)
-            matches = re.match(regex, output[0])
-            if matches:
-                return matches.group(1)
-
-
-class Md5File(FactBase):
+class Md5File(HashFileFactBase, digits=32, cmds=["md5sum", "md5"]):
     """
     Returns an MD5 hash of a file, or ``None`` if the file does not exist.
     """
 
-    _regexes = [
-        r"^([a-zA-Z0-9]{32})\s+%s$",
-        r"^SHA256\s+\(%s\)\s+=\s+([a-zA-Z0-9]{32})$",
-    ]
-
-    def command(self, path):
-        self.path = path
-        return make_formatted_string_command(
-            "test -e {0} && ( md5sum {0} 2> /dev/null || md5 {0} ) || true",
-            QuoteString(path),
-        )
-
-    def process(self, output):
-        for regex in self._regexes:
-            regex = regex % re.escape(self.path)
-            matches = re.match(regex, output[0])
-            if matches:
-                return matches.group(1)
-
 
 class FindInFile(FactBase):
     """
@@ -354,3 +329,81 @@ class FindDirectories(FindFilesBase):
     """
 
     type_flag = "d"
+
+
+class Flags(FactBase):
+    """
+    Returns a list of the file flags set for the specified file or directory.
+    """
+
+    requires_command = "chflags"  # don't try to retrieve them if we can't set them
+
+    def command(self, path):
+        return make_formatted_string_command(
+            "! test -e {0} || stat -f %Sf {0}",
+            QuoteString(path),
+        )
+
+    def process(self, output):
+
+        return [flag for flag in output[0].split(",") if len(flag) > 0] if len(output) == 1 else []
+
+
+MARKER_DEFAULT = "# {mark} PYINFRA BLOCK"
+MARKER_BEGIN_DEFAULT = "BEGIN"
+MARKER_END_DEFAULT = "END"
+EXISTS = "__pyinfra_exists_"
+MISSING = "__pyinfra_missing_"
+
+
+class Block(FactBase):
+    """
+    Returns a (possibly empty) list of the lines found between the markers.
+
+    .. code:: python
+
+        [
+            "xray: one",
+            "alpha: two"
+        ]
+
+    If the ``path`` doesn't exist
+        returns ``None``
+
+    If the ``path`` exists but the markers are not found
+        returns ``[]``
+    """
+
+    # if markers aren't found, awk will return 0 and produce no output but we need to
+    # distinguish between "markers not found" and "markers found but nothing between them"
+    # for the former we use the empty list (created the call to default) and for the latter
+    # the list with a single empty string.
+    default = list
+
+    def command(self, path, marker=None, begin=None, end=None):
+
+        self.path = path
+        start = (marker or MARKER_DEFAULT).format(mark=begin or MARKER_BEGIN_DEFAULT)
+        end = (marker or MARKER_DEFAULT).format(mark=end or MARKER_END_DEFAULT)
+        if start == end:
+            raise ValueError(f"delimiters for block must be different but found only '{start}'")
+
+        backstop = make_formatted_string_command(
+            "(find {0} -type f > /dev/null && echo {1} || echo {2} )",
+            QuoteString(path),
+            QuoteString(f"{EXISTS}{path}"),
+            QuoteString(f"{MISSING}{path}"),
+        )
+        # m_f_s_c inserts blanks in unfortunate places, e.g. after first slash
+        cmd = make_formatted_string_command(
+            f"awk \\'/{end}/{{{{f=0}}}} f; /{start}/{{{{f=1}}}}\\' {{0}} || {backstop}",
+            QuoteString(path),
+        )
+        return cmd
+
+    def process(self, output):
+        if output and (output[0] == f"{EXISTS}{self.path}"):
+            return []
+        if output and (output[0] == f"{MISSING}{self.path}"):
+            return None
+        return output
