@@ -163,8 +163,8 @@ class MysqlUsers(MysqlFactBase):
 
 
 MYSQL_GRANT_REGEX = (
-    r"^GRANT ([A-Z,\s]+) ON ((?:\*|`[a-z_\\]+`)(?:\.\*|'[a-z_]+')) "
-    r"TO `[A-Z0-9a-z_\-]+`@`[A-Z0-9a-z_\.\-]+`(.*)"
+    r"^GRANT ([A-Z,\s]+) ON ((?:\*|`[a-z_\\]+`)\.(?:\*|`[a-z_]+`)) "
+    r"TO `[A-Z0-9a-z_\-]+`@`(?:%|[A-Z0-9a-z_\.\-]+)`(.*)"
 )
 
 
@@ -199,7 +199,7 @@ class MysqlUserGrants(MysqlFactBase):
     ):
         self.mysql_command = 'SHOW GRANTS FOR "{0}"@"{1}"'.format(user, hostname)
 
-        return super(MysqlUserGrants, self).command(
+        return super().command(
             mysql_user,
             mysql_password,
             mysql_host,
