@@ -4,7 +4,7 @@ from .util.packaging import parse_packages
 
 
 class PkgPackages(FactBase):
-    '''
+    """
     Returns a dict of installed pkg packages:
 
     .. code:: python
@@ -12,10 +12,10 @@ class PkgPackages(FactBase):
         {
             "package_name": ["version"],
         }
-    '''
+    """
 
-    command = 'pkg info || pkg_info || true'
-    regex = r'^([a-zA-Z0-9_\-\+]+)\-([0-9a-z\.]+)'
+    command = "pkg info || pkg_info || true"
+    regex = r"^([a-zA-Z0-9_\-\+]+)\-([0-9a-z\.]+)"
     default = dict
 
     def process(self, output):
