@@ -5,11 +5,12 @@ from datetime import datetime
 from tempfile import mkdtemp
 
 from dateutil.parser import parse as parse_date
+from distro import distro
 
 from pyinfra.api import FactBase, ShortFactBase
 from pyinfra.api.util import try_int
 
-from .util.distro import get_distro_info
+ISO_DATE_FORMAT = "%Y-%m-%dT%H:%M:%S%z"
 
 
 class User(FactBase):
@@ -119,12 +120,12 @@ class Date(FactBase):
     Returns the current datetime on the server.
     """
 
-    command = "LANG=C LC_TIME=en_US.UTF-8 date"
+    command = f"date +'{ISO_DATE_FORMAT}'"
     default = datetime.now
 
     @staticmethod
     def process(output):
-        return parse_date(output[0])
+        return datetime.strptime(output[0], ISO_DATE_FORMAT)
 
 
 class MacosVersion(FactBase):
@@ -416,6 +417,7 @@ class Users(FactBase):
                 "uid": user_id,
                 "gid": main_user_group_id,
                 "lastlog": last_login_time,
+                "password": encrypted_password,
             },
         }
     """
@@ -423,8 +425,10 @@ class Users(FactBase):
     command = """
         for i in `cat /etc/passwd | cut -d: -f1`; do
             ENTRY=`grep ^$i: /etc/passwd`;
-            LASTLOG=`lastlog -u $i | grep ^$i` | tr -s ' ';
-            echo "$ENTRY|`id -gn $i`|`id -Gn $i`|$LASTLOG";
+            LASTLOG_RAW=`(lastlog -u $i 2> /dev/null || lastlogin $i 2> /dev/null)`;
+            LASTLOG=`echo $LASTLOG_RAW | grep ^$i | tr -s ' '`;
+            PASSWORD=`grep ^$i: /etc/shadow | cut -d: -f2`;
+            echo "$ENTRY|`id -gn $i`|`id -Gn $i`|$LASTLOG|$PASSWORD";
         done
     """.strip()
 
@@ -435,7 +439,7 @@ class Users(FactBase):
         rex = r"[A-Z][a-z]{2} [A-Z][a-z]{2} {1,2}\d+ .+$"
 
         for line in output:
-            entry, group, user_groups, lastlog = line.split("|")
+            entry, group, user_groups, lastlog, password = line.rsplit("|", 4)
 
             if entry:
                 # Parse out the comment/home/shell
@@ -468,6 +472,7 @@ class Users(FactBase):
                     "gid": int(entries[3]),
                     "lastlog": raw_login_time,
                     "login_time": login_time,
+                    "password": password,
                 }
 
         return users
@@ -542,10 +547,16 @@ class LinuxDistribution(FactBase):
             os.mkdir(temp_etc_dir)
 
             for filename, content in parts.items():
-                with open(os.path.join(temp_etc_dir, os.path.basename(filename)), "w") as fp:
+                with open(
+                    os.path.join(temp_etc_dir, os.path.basename(filename)), "w", encoding="utf-8"
+                ) as fp:
                     fp.write(content)
 
-            parsed = get_distro_info(temp_root)
+            parsed = distro.LinuxDistribution(
+                root_dir=temp_root,
+                include_lsb=False,
+                include_uname=False,
+            )
 
             release_meta = {key.upper(): value for key, value in parsed.os_release_info().items()}
             # Distro 1.7+ adds this, breaking tests
@@ -650,3 +661,104 @@ class HasGui(ShortFactBase):
     @staticmethod
     def process_data(data):
         return len(data) > 0
+
+
+class Locales(FactBase):
+    """
+    Returns installed locales on the target host.
+
+    .. code:: python
+
+        ["C.UTF-8", "en_US.UTF-8"]
+    """
+
+    command = "locale -a"
+    requires_command = "locale"
+    default = list
+
+    def process(self, output):
+        # replace utf8 with UTF-8 to match names in /etc/locale.gen
+        # return a list of enabled locales
+        return [line.replace("utf8", "UTF-8") for line in output]
+
+
+class SecurityLimits(FactBase):
+    """
+    Returns a list of security limits on the target host.
+
+    .. code:: python
+
+        [
+            {
+                "domain": "*",
+                "limit_type": "soft",
+                "item": "nofile",
+                "value": "1048576"
+            },
+            {
+                "domain": "*",
+                "limit_type": "hard",
+                "item": "nofile",
+                "value": "1048576"
+            },
+            {
+                "domain": "root",
+                "limit_type": "soft",
+                "item": "nofile",
+                "value": "1048576"
+            },
+            {
+                "domain": "root",
+                "limit_type": "hard",
+                "item": "nofile",
+                "value": "1048576"
+            },
+            {
+                "domain": "*",
+                "limit_type": "soft",
+                "item": "memlock",
+                "value": "unlimited"
+            },
+            {
+                "domain": "*",
+                "limit_type": "hard",
+                "item": "memlock",
+                "value": "unlimited"
+            },
+            {
+                "domain": "root",
+                "limit_type": "soft",
+                "item": "memlock",
+                "value": "unlimited"
+            },
+            {
+                "domain": "root",
+                "limit_type": "hard",
+                "item": "memlock",
+                "value": "unlimited"
+            }
+        ]
+    """
+
+    command = "cat /etc/security/limits.conf"
+    default = list
+
+    def process(self, output):
+        limits = []
+
+        for line in output:
+            if line.startswith("#") or not len(line.strip()):
+                continue
+
+            domain, limit_type, item, value = line.split()
+
+            limits.append(
+                {
+                    "domain": domain,
+                    "limit_type": limit_type,
+                    "item": item,
+                    "value": value,
+                },
+            )
+
+        return limits
