@@ -4,7 +4,7 @@ from pyinfra.api import FactBase
 
 
 class InitdStatus(FactBase):
-    '''
+    """
     Low level check for every /etc/init.d/* script. Unfortunately many of these
     mishehave and return exit status 0 while also displaying the help info/not
     offering status support.
@@ -13,9 +13,9 @@ class InitdStatus(FactBase):
 
     Expected codes found at:
         http://refspecs.linuxbase.org/LSB_3.1.0/LSB-Core-generic/LSB-Core-generic/iniscrptact.html
-    '''
+    """
 
-    command = '''
+    command = """
         for SERVICE in `ls /etc/init.d/`; do
             _=`cat /etc/init.d/$SERVICE | grep "### BEGIN INIT INFO"`
 
@@ -24,9 +24,9 @@ class InitdStatus(FactBase):
                 echo "$SERVICE=$?"
             fi
         done
-    '''
+    """
 
-    regex = r'([a-zA-Z0-9\-]+)=([0-9]+)'
+    regex = r"([a-zA-Z0-9\-]+)=([0-9]+)"
     default = dict
 
     def process(self, output):
