@@ -10,7 +10,7 @@ from pyinfra.connectors.util import run_local_process, split_combined_output
 from pyinfra.context import ctx_state
 
 
-def include(filename):
+def include(filename: str):
     """
     Executes a local python file within the ``pyinfra.state.cwd``
     directory.
@@ -21,7 +21,7 @@ def include(filename):
 
     filename = get_file_path(state, filename)
 
-    logger.debug("Including local file: {0}".format(filename))
+    logger.debug("Including local file: %s", filename)
 
     config_state = config.get_current_state()
 
@@ -39,22 +39,23 @@ def include(filename):
         # One potential solution to the above is to add local as an actual
         # module, ie `pyinfra.operations.local`.
 
-    except Exception as e:
-        raise PyinfraError(
-            "Could not include local file: {0}:\n{1}".format(filename, e),
-        )
-
     finally:
         config.set_current_state(config_state)
 
 
-def shell(commands, splitlines=False, ignore_errors=False, print_output=False, print_input=False):
+def shell(
+    commands,
+    splitlines: bool = False,
+    ignore_errors: bool = False,
+    print_output: bool = False,
+    print_input: bool = False,
+):
     """
     Subprocess based implementation of pyinfra/api/ssh.py's ``run_shell_command``.
 
     Args:
         commands (string, list): command or list of commands to execute
-        spltlines (bool): optionally have the output split by lines
+        splitlines (bool): optionally have the output split by lines
         ignore_errors (bool): ignore errors when executing these commands
     """
 
