@@ -4,7 +4,6 @@ The files operations handles filesystem state, file uploads and template generat
 
 import os
 import posixpath
-import re
 import sys
 import traceback
 from datetime import timedelta
@@ -20,6 +19,7 @@ from pyinfra.api import (
     FileUploadCommand,
     OperationError,
     OperationTypeError,
+    OperationValueError,
     QuoteString,
     RsyncCommand,
     StringCommand,
@@ -34,10 +34,15 @@ from pyinfra.api.util import (
     memoize,
 )
 from pyinfra.facts.files import (
+    MARKER_BEGIN_DEFAULT,
+    MARKER_DEFAULT,
+    MARKER_END_DEFAULT,
+    Block,
     Directory,
     File,
     FindFiles,
     FindInFile,
+    Flags,
     Link,
     Md5File,
     Sha1File,
@@ -46,7 +51,7 @@ from pyinfra.facts.files import (
 from pyinfra.facts.server import Date, Which
 
 from .util import files as file_utils
-from .util.files import ensure_mode_int, get_timestamp, sed_replace, unix_path_join
+from .util.files import adjust_regex, ensure_mode_int, get_timestamp, sed_replace, unix_path_join
 
 
 @operation(
@@ -65,6 +70,7 @@ def download(
     md5sum=None,
     headers=None,
     insecure=False,
+    proxy=None,
 ):
     """
     Download files from remote locations using ``curl`` or ``wget``.
@@ -81,6 +87,7 @@ def download(
     + md5sum: md5 hash to checksum the downloaded file against
     + headers: optional dictionary of headers to set for the HTTP request
     + insecure: disable SSL verification for the HTTP request
+    + proxy: simple HTTP proxy through which we can download files, form `http://<yourproxy>:<port>`
 
     **Example:**
 
@@ -136,7 +143,14 @@ def download(
         temp_file = state.get_temp_filename(dest)
 
         curl_args = ["-sSLf"]
+
         wget_args = ["-q"]
+
+        if proxy:
+            curl_args.append(f"--proxy {proxy}")
+            wget_args.append("-e use_proxy=yes")
+            wget_args.append(f"-e http_proxy={proxy}")
+
         if insecure:
             curl_args.append("--insecure")
             wget_args.append("--no-check-certificate")
@@ -241,6 +255,7 @@ def line(
     interpolate_variables=False,
     escape_regex_characters=False,
     assume_present=False,
+    ensure_newline=False,
 ):
     """
     Ensure lines in files using grep to locate and sed to replace.
@@ -254,6 +269,7 @@ def line(
     + interpolate_variables: whether to interpolate variables in ``replace``
     + assume_present: whether to assume a matching line already exists in the file
     + escape_regex_characters: whether to escape regex characters from the matching line
+    + ensure_newline: ensures that the appended line is on a new line
 
     Regex line matching:
         Unless line matches a line (starts with ^, ends $), pyinfra will wrap it such that
@@ -273,6 +289,11 @@ def line(
         If ``line`` is not in the file but we want it (``present`` set to ``True``), then
         it will be append to the end of the file.
 
+    Ensure new line:
+        This will ensure that the ``line`` being appended is always on a seperate new
+        line in case the file doesn't end with a newline character.
+
+
     **Examples:**
 
     .. code:: python
@@ -280,12 +301,12 @@ def line(
         # prepare to do some maintenance
         maintenance_line = "SYSTEM IS DOWN FOR MAINTENANCE"
         files.line(
-            name="Add the down-for-maintence line in /etc/motd",
+            name="Add the down-for-maintenance line in /etc/motd",
             path="/etc/motd",
             line=maintenance_line,
         )
 
-        # Then, after the mantenance is done, remove the maintenance line
+        # Then, after the maintenance is done, remove the maintenance line
         files.line(
             name="Remove the down-for-maintenance line in /etc/motd",
             path="/etc/motd",
@@ -320,17 +341,17 @@ def line(
         )
     """
 
-    match_line = line
-
-    if escape_regex_characters:
-        match_line = re.sub(r"([\.\\\+\*\?\[\^\]\$\(\)\{\}\-])", r"\\\1", match_line)
-
-    # Ensure we're matching a whole line, note: match may be a partial line so we
-    # put any matches on either side.
-    if not match_line.startswith("^"):
-        match_line = "^.*{0}".format(match_line)
-    if not match_line.endswith("$"):
-        match_line = "{0}.*$".format(match_line)
+    match_line = adjust_regex(line, escape_regex_characters)
+    #
+    # if escape_regex_characters:
+    #     match_line = re.sub(r"([\.\\\+\*\?\[\^\]\$\(\)\{\}\-])", r"\\\1", match_line)
+    #
+    # # Ensure we're matching a whole line, note: match may be a partial line so we
+    # # put any matches on either side.
+    # if not match_line.startswith("^"):
+    #     match_line = "^.*{0}".format(match_line)
+    # if not match_line.endswith("$"):
+    #     match_line = "{0}.*$".format(match_line)
 
     # Is there a matching line in this file?
     if assume_present:
@@ -351,11 +372,18 @@ def line(
         replace = ""
 
     # Save commands for re-use in dynamic script when file not present at fact stage
-    echo_command = make_formatted_string_command(
-        "echo {0} >> {1}",
-        '"{0}"'.format(line) if interpolate_variables else QuoteString(line),
-        QuoteString(path),
-    )
+    if ensure_newline:
+        echo_command = make_formatted_string_command(
+            "( [ $(tail -c1 {1} | wc -l) -eq 0 ] && echo ; echo {0} ) >> {1}",
+            '"{0}"'.format(line) if interpolate_variables else QuoteString(line),
+            QuoteString(path),
+        )
+    else:
+        echo_command = make_formatted_string_command(
+            "echo {0} >> {1}",
+            '"{0}"'.format(line) if interpolate_variables else QuoteString(line),
+            QuoteString(path),
+        )
 
     if backup:
         backup_filename = "{0}.{1}".format(path, get_timestamp())
@@ -708,7 +736,8 @@ def rsync(src, dest, flags=["-ax", "--delete"]):
 
     .. important::
         The ``files.rsync`` operation is in alpha, and only supported using SSH
-        or ``@local`` connectors.
+        or ``@local`` connectors. When using the SSH connector, rsync will automatically use the
+        StrictHostKeyChecking setting, config and known_hosts file (when specified).
 
     .. caution::
         When using SSH, the ``files.rsync`` operation only supports the ``sudo`` and ``sudo_user``
@@ -1528,3 +1557,281 @@ def directory(
 
         if not changed:
             host.noop("directory {0} already exists".format(path))
+
+
+@operation(pipeline_facts={"flags": "path"})
+def flags(path, flags=None, present=True):
+    """
+    Set/clear file flags.
+
+    + path: path of the remote folder
+    + flags: a list of the file flags to be set or cleared
+    + present: whether the flags should be set or cleared
+
+    **Examples:**
+
+    .. code:: python
+
+        files.flags(
+            name="Ensure ~/Library is visible in the GUI",
+            path="~/Library",
+            flags="hidden",
+            present=False
+        )
+
+        files.directory(
+            name="Ensure no one can change these files",
+            path="/something/very/important",
+            flags=["uchg", "schg"],
+            present=True,
+            _sudo=True
+        )
+    """
+    flags = flags or []
+    if not isinstance(flags, list):
+        flags = [flags]
+
+    if len(flags) == 0:
+        host.noop(f"no changes requested to flags for '{path}'")
+    else:
+        current_set = set(host.get_fact(Flags, path=path))
+        to_change = list(set(flags) - current_set) if present else list(current_set & set(flags))
+
+        if len(to_change) > 0:
+            prefix = "" if present else "no"
+            new_flags = ",".join([prefix + flag for flag in sorted(to_change)])
+            yield StringCommand("chflags", new_flags, QuoteString(path))
+            host.create_fact(
+                Flags,
+                kwargs={"path": path},
+                data=list(current_set | set(to_change))
+                if present
+                else list(current_set - set(to_change)),
+            )
+        else:
+            host.noop(
+                f'\'{path}\' already has \'{",".join(flags)}\' {"set" if present else "clear"}',
+            )
+
+
+@operation(
+    pipeline_facts={"file": "path"},
+)
+def block(
+    path,
+    content=None,
+    present=True,
+    line=None,
+    backup=False,
+    escape_regex_characters=False,
+    try_prevent_shell_expansion=False,
+    before=False,
+    after=False,
+    marker=None,
+    begin=None,
+    end=None,
+):
+    """
+    Ensure content, surrounded by the appropriate markers, is present (or not) in the file.
+
+    + path: target remote file
+    + content: what should be present in the file (between markers).
+    + present: whether the content should be present in the file
+    + before: should the content be added before ``line`` if it doesn't exist
+    + after: should the content be added after ``line`` if it doesn't exist
+    + line: regex before or after which the content should be added if it doesn't exist.
+    + backup: whether to backup the file (see ``files.line``). Default False.
+    + escape_regex_characters: whether to escape regex characters from the matching line
+    + try_prevent_shell_expansion: tries to prevent shell expanding by values like `$`
+    + marker: the base string used to mark the text.  Default is ``# {mark} PYINFRA BLOCK``
+    + begin: the value for ``{mark}`` in the marker before the content. Default is ``BEGIN``
+    + end: the value for ``{mark}`` in the marker after the content. Default is ``END``
+
+    Content appended if ``line`` not found in the file
+        If ``content`` is not in the file but is required (``present=True``) and ``line`` is not
+        found in the file, ``content`` (surrounded by markers) will be appended to the file.  The
+        file is created if necessary.
+
+    Content prepended or appended if ``line`` not specified
+        If ``content`` is not in the file but is required and ``line`` was not provided the content
+        will either be  prepended to the file (if both ``before`` and ``after``
+        are ``True``) or appended to the file (if both are ``False``).
+
+    Removal ignores ``content`` and ``line``
+
+    Preventing shell expansion works by wrapping the content in '`' before passing to `awk`.
+    WARNING: This will break if the content contains raw single quotes.
+
+    **Examples:**
+
+    .. code:: python
+
+        # add entry to /etc/host
+        files.block(
+            name="add IP address for red server",
+            path="/etc/hosts",
+            content="10.0.0.1 mars-one",
+            before=True,
+            regex=".*localhost",
+        )
+
+        # have two entries in /etc/host
+        files.block(
+            name="add IP address for red server",
+            path="/etc/hosts",
+            content="10.0.0.1 mars-one\\n10.0.0.2 mars-two",
+            before=True,
+            regex=".*localhost",
+        )
+
+        # remove marked entry from /etc/hosts
+        files.block(
+            name="remove all 10.* addresses from /etc/hosts",
+            path="/etc/hosts",
+            present=False
+        )
+
+        # add out of date warning to web page
+        files.block(
+            name="add out of date warning to web page",
+            path="/var/www/html/something.html",
+            content= "<p>Warning: this page is out of date.</p>",
+            regex=".*<body>.*",
+            after=True
+            marker="<!-- {mark} PYINFRA BLOCK -->",
+        )
+
+        # put complex alias into .zshrc
+        files.block(
+            path="/home/user/.zshrc",
+            content="eval $(thefuck -a)",
+            try_prevent_shell_expansion=True,
+            marker="## {mark} ALIASES ##"
+        )
+    """
+
+    logger.warning("The `files.block` operation is currently in beta!")
+
+    mark_1 = (marker or MARKER_DEFAULT).format(mark=begin or MARKER_BEGIN_DEFAULT)
+    mark_2 = (marker or MARKER_DEFAULT).format(mark=end or MARKER_END_DEFAULT)
+
+    # standard awk doesn't have an "in-place edit" option so we write to a tempfile and
+    # if edits were successful move to dest i.e. we do: <out_prep> ... do some work ... <real_out>
+    q_path = QuoteString(path)
+    out_prep = 'OUT="$(TMPDIR=/tmp mktemp -t pyinfra.XXXXXX)" && '
+    if backup:
+        out_prep = StringCommand(
+            "cp",
+            q_path,
+            QuoteString(f"{path}.{get_timestamp()}"),
+            "&&",
+            out_prep,
+        )
+    real_out = StringCommand(
+        "chmod $(stat -c %a",
+        q_path,
+        "2>/dev/null || stat -f %Lp",
+        q_path,
+        ") $OUT && ",
+        '(chown $(stat -c "%U:%G"',
+        q_path,
+        "2>/dev/null) $OUT || ",
+        'chown -n $(stat -f "%u:%g"',
+        q_path,
+        ') $OUT)  && mv "$OUT"',
+        q_path,
+    )
+
+    current = host.get_fact(Block, path=path, marker=marker, begin=begin, end=end)
+    cmd = None
+    if present:
+        if not content:
+            raise OperationValueError("'content' must be supplied when 'present' == True")
+        if line:
+            if before == after:
+                raise OperationValueError(
+                    "only one of 'before' or 'after' used when 'line` is specified"
+                )
+        elif before != after:
+            raise OperationValueError(
+                "'line' must be supplied or 'before' and 'after' must be equal"
+            )
+        if isinstance(content, str):
+            # convert string to list of lines
+            content = content.split("\n")
+        if try_prevent_shell_expansion and any("'" in line for line in content):
+            logger.warning("content contains single quotes, shell expansion prevention may fail")
+
+        the_block = "\n".join([mark_1, *content, mark_2])
+
+        if (current is None) or ((current == []) and (before == after)):
+            # a) no file or b) file but no markers and we're adding at start or end. Both use 'cat'
+            redirect = ">" if (current is None) else ">>"
+            stdin = "- " if ((current == []) and before) else ""
+            # here = hex(random.randint(0, 2147483647))
+            here = "PYINFRAHERE"
+            cmd = StringCommand(
+                f"cat {stdin}{redirect}",
+                q_path,
+                f"<<{here}" if not try_prevent_shell_expansion else f"<<'{here}'",
+                f"\n{the_block}\n{here}",
+            )
+        elif current == []:  # markers not found and have a pattern to match (not start or end)
+            regex = adjust_regex(line, escape_regex_characters)
+            print_before = "{ print }" if before else ""
+            print_after = "{ print }" if after else ""
+            prog = (
+                'awk \'BEGIN {x=ARGV[2]; ARGV[2]=""} '
+                f"{print_after} f!=1 && /{regex}/ {{ print x; f=1}} "
+                f"END {{if (f==0) print ARGV[2] }} {print_before}'"
+            )
+            cmd = StringCommand(
+                out_prep,
+                prog,
+                q_path,
+                f'"{the_block}"' if not try_prevent_shell_expansion else f"'{the_block}'",
+                "> $OUT &&",
+                real_out,
+            )
+        else:
+            if (len(current) != len(content)) or (
+                not all(lines[0] == lines[1] for lines in zip(content, current))
+            ):  # marked_block found but text is different
+                prog = (
+                    'awk \'BEGIN {{f=1; x=ARGV[2]; ARGV[2]=""}}'
+                    f"/{mark_1}/ {{print; print x; f=0}} /{mark_2}/ {{print; f=1; next}} f'"
+                )
+                cmd = StringCommand(
+                    out_prep,
+                    prog,
+                    q_path,
+                    '"' + "\n".join(content) + '"'
+                    if not try_prevent_shell_expansion
+                    else "'" + "\n".join(content) + "'",
+                    "> $OUT &&",
+                    real_out,
+                )
+            else:
+                host.noop("content already present")
+
+        if cmd:
+            yield cmd
+            host.create_fact(
+                Block,
+                kwargs={"path": path, "marker": marker, "begin": begin, "end": end},
+                data=content,
+            )
+    else:  # remove the marked_block
+        if content:
+            logger.warning("'content' ignored when removing a marked_block")
+        if current is None:
+            host.noop("no remove required: file did not exist")
+        elif current == []:
+            host.noop("no remove required: markers not found")
+        else:
+            cmd = f"awk '/{mark_1}/,/{mark_2}/ {{next}} 1'"
+            yield StringCommand(out_prep, cmd, q_path, "> $OUT &&", real_out)
+            host.delete_fact(
+                Block,
+                kwargs={"path": path, "marker": marker, "begin": begin, "end": end},
+            )
