@@ -14,7 +14,7 @@ See the example/mysql.py
 """
 
 from pyinfra import host
-from pyinfra.api import MaskString, OperationError, StringCommand, operation
+from pyinfra.api import MaskString, OperationError, QuoteString, StringCommand, operation
 from pyinfra.facts.mysql import (
     MysqlDatabases,
     MysqlUserGrants,
@@ -353,7 +353,7 @@ def database(
     if not present:
         if is_present:
             yield make_execute_mysql_command(
-                "DROP DATABASE {0}".format(database),
+                "DROP DATABASE `{0}`".format(database),
                 user=mysql_user,
                 password=mysql_password,
                 host=mysql_host,
@@ -366,7 +366,7 @@ def database(
 
     # We want the database but it doesn't exist
     if present and not is_present:
-        sql_bits = ["CREATE DATABASE {0}".format(database)]
+        sql_bits = ["CREATE DATABASE `{0}`".format(database)]
 
         if collate:
             sql_bits.append("COLLATE {0}".format(collate))
@@ -475,7 +475,7 @@ def privileges(
     else:
         user_grants[database_table] = set()
 
-    def handle_privileges(action, target, privileges_to_apply):
+    def handle_privileges(action, target, privileges_to_apply, with_statement=None):
         command = (
             "{action} {privileges} " "ON {database}.{table} " '{target} "{user}"@"{user_hostname}"'
         ).format(
@@ -488,6 +488,9 @@ def privileges(
             user_hostname=user_hostname,
         )
 
+        if with_statement:
+            command += " WITH {with_statement}".format(with_statement=with_statement)
+
         yield make_execute_mysql_command(
             command,
             user=mysql_user,
@@ -501,14 +504,30 @@ def privileges(
     # Find / grant any privileges that we want but do not exist
     privileges_to_grant = privileges - existing_privileges
 
-    # No privilege and we want it
+    user_grants[database_table] -= privileges_to_revoke
+    user_grants[database_table].update(privileges_to_grant)
+
     if privileges_to_grant:
-        yield from handle_privileges("GRANT", "TO", privileges_to_grant)
-        user_grants[database_table].update(privileges_to_grant)
+        # We will grant something on this table, no need to revoke "USAGE" as it will be overridden
+        privileges_to_revoke.discard("USAGE")
 
     if privileges_to_revoke:
-        yield from handle_privileges("REVOKE", "FROM", privileges_to_revoke)
-        user_grants[database_table] -= privileges_to_revoke
+        if {"ALL", "GRANT OPTION"} == privileges_to_revoke:
+            # This specific case is identified as the alternative syntax for revoke (without ON).
+            # To avoid this problem, revoke in 2 steps.
+            yield from handle_privileges("REVOKE", "FROM", {"GRANT OPTION"})
+            yield from handle_privileges("REVOKE", "FROM", {"ALL PRIVILEGES"})
+        else:
+            yield from handle_privileges("REVOKE", "FROM", privileges_to_revoke)
+
+    if privileges_to_grant:
+        if {"ALL", "GRANT OPTION"} == privileges_to_grant:
+            # ALL must be named by itself and cannot be specified along with other privileges
+            # The only exception for us is GRANT OPTION but as a WITH clause
+            # So handle this case and let the other ones fail
+            yield from handle_privileges("GRANT", "TO", {"ALL PRIVILEGES"}, "GRANT OPTION")
+        else:
+            yield from handle_privileges("GRANT", "TO", privileges_to_grant)
 
     if privileges_to_grant or privileges_to_revoke:
         if flush:
@@ -595,7 +614,7 @@ def load(
         )
     """
 
-    yield "{0} < {1}".format(
+    commands_bits = [
         make_mysql_command(
             database=database,
             user=mysql_user,
@@ -603,5 +622,7 @@ def load(
             host=mysql_host,
             port=mysql_port,
         ),
-        src,
-    )
+        "<",
+        QuoteString(src),
+    ]
+    yield StringCommand(*commands_bits)
