@@ -9,16 +9,17 @@ from itertools import filterfalse, tee
 from os import path
 from time import sleep
 
-from pyinfra import host, state
+from pyinfra import host, logger, state
 from pyinfra.api import FunctionCommand, OperationError, StringCommand, operation
 from pyinfra.api.util import try_int
 from pyinfra.connectors.util import remove_any_sudo_askpass_file
-from pyinfra.facts.files import Directory
+from pyinfra.facts.files import Directory, FindInFile, Link
 from pyinfra.facts.server import (
     Crontab,
     Groups,
     Hostname,
     KernelModules,
+    Locales,
     Mounts,
     Os,
     Sysctl,
@@ -98,9 +99,15 @@ def reboot(delay=10, interval=1, reboot_timeout=300):
 
     yield FunctionCommand(wait_and_reconnect, (), {})
 
+    # On certain systems sudo files are lost on reboot
+    def clean_sudo_info(state, host):
+        host.connector_data["sudo_askpass_path"] = None
+
+    yield FunctionCommand(clean_sudo_info, (), {})
+
 
 @operation(is_idempotent=False)
-def wait(port=None):
+def wait(port: int):
     """
     Waits for a port to come active on the target machine. Requires netstat, checks every
     second.
@@ -155,11 +162,12 @@ def shell(commands):
 
 
 @operation(is_idempotent=False)
-def script(src):
+def script(src, args=()):
     """
     Upload and execute a local script on the remote host.
 
     + src: local script filename to upload & execute
+    + args: iterable to pass as arguments to the script
 
     **Example:**
 
@@ -170,17 +178,24 @@ def script(src):
             name="Hello",
             src="files/hello.bash",
         )
+
+        # Example passing arguments to the script
+        server.script(
+            name="Hello",
+            src="files/hello.bash",
+            args=("do-something", "with-this"),
+        )
     """
 
-    temp_file = state.get_temp_filename(src)
+    temp_file = state.get_temp_filename()
     yield from files.put(src, temp_file)
 
     yield chmod(temp_file, "+x")
-    yield temp_file
+    yield StringCommand(temp_file, *args)
 
 
 @operation(is_idempotent=False)
-def script_template(src, **data):
+def script_template(src, args=(), **data):
     """
     Generate, upload and execute a local script template on the remote host.
 
@@ -206,7 +221,7 @@ def script_template(src, **data):
     yield from files.template(src, temp_file, **data)
 
     yield chmod(temp_file, "+x")
-    yield temp_file
+    yield StringCommand(temp_file, *args)
 
 
 @operation
@@ -269,8 +284,10 @@ def mount(
     path,
     mounted=True,
     options=None,
+    device=None,
+    fs_type=None,
     # TODO: do we want to manage fstab here?
-    # update_fstab=False, device=None, fs_type=None,
+    # update_fstab=False,
 ):
     """
     Manage mounted filesystems.
@@ -287,7 +304,6 @@ def mount(
         This operation does not attempt to modify the on disk fstab file - for
         that you should use the `files.line operation <./files.html#files-line>`_.
     """
-
     options = options or []
     options_string = ",".join(options)
 
@@ -296,10 +312,17 @@ def mount(
 
     # Want mount but don't have?
     if mounted and not is_mounted:
-        yield "mount{0} {1}".format(
-            " -o {0}".format(options_string) if options_string else "",
-            path,
-        )
+        args = []
+        if fs_type:
+            args.extend(["-t", fs_type])
+        if options_string:
+            args.extend(["-o", options_string])
+        if device:
+            args.append(device)
+        args.append(path)
+
+        yield StringCommand("mount", *args)
+        # Should we update facts with fs_type, device, etc?
         mounts[path] = {"options": options}
 
     # Want no mount but mounted?
@@ -471,10 +494,16 @@ def service(
     elif host.get_fact(Which, command="initctl"):
         service_operation = upstart.service
 
-    elif host.get_fact(Directory, path="/etc/init.d"):
+    elif (
+        host.get_fact(Which, command="service")
+        or host.get_fact(Link, path="/etc/init.d")
+        or host.get_fact(Directory, path="/etc/init.d")
+    ):
         service_operation = sysvinit.service
 
-    elif host.get_fact(Directory, path="/etc/rc.d"):
+    # NOTE: important that we are not Linux here because /etc/rc.d will exist but checking it's
+    # contents may trigger things (like a reboot: https://github.com/Fizzadar/pyinfra/issues/819)
+    elif host.get_fact(Os) != "Linux" and host.get_fact(Directory, path="/etc/rc.d"):
         service_operation = bsdinit.service
 
     else:
@@ -750,6 +779,7 @@ def group(group, present=True, system=False, gid=None):
     + group: name of the group to ensure
     + present: whether the group should be present or not
     + system: whether to create a system group
+    + gid: use a specific groupid number
 
     System users:
         System users don't exist on BSD, so the argument is ignored for BSD targets.
@@ -772,11 +802,15 @@ def group(group, present=True, system=False, gid=None):
     """
 
     groups = host.get_fact(Groups)
+    os_type = host.get_fact(Os)
     is_present = group in groups
 
     # Group exists but we don't want them?
     if not present and is_present:
-        yield "groupdel {0}".format(group)
+        if os_type == "FreeBSD":
+            yield "pw groupdel -n {0}".format(group)
+        else:
+            yield "groupdel {0}".format(group)
         groups.remove(group)
 
     # Group doesn't exist and we want it?
@@ -787,17 +821,23 @@ def group(group, present=True, system=False, gid=None):
         if system and "BSD" not in host.get_fact(Os):
             args.append("-r")
 
-        args.append(group)
+        if os_type == "FreeBSD":
+            args.append("-n {0}".format(group))
+        else:
+            args.append(group)
 
         if gid:
-            args.append("--gid {0}".format(gid))
+            if os_type == "FreeBSD":
+                args.append("-g {0}".format(gid))
+            else:
+                args.append("--gid {0}".format(gid))
 
         # Groups are often added by other operations (package installs), so check
         # for the group at runtime before adding.
-        yield "grep '^{0}:' /etc/group || groupadd {1}".format(
-            group,
-            " ".join(args),
-        )
+        group_add_command = "groupadd"
+        if os_type == "FreeBSD":
+            group_add_command = "pw groupadd"
+        yield "grep '^{0}:' /etc/group || {2} {1}".format(group, " ".join(args), group_add_command)
         groups.append(group)
 
 
@@ -847,7 +887,7 @@ def user_authorized_keys(
 
         if path.exists(try_path):
             with open(try_path, "r") as f:
-                return f.read()
+                return f.read().strip()
 
         return key
 
@@ -893,7 +933,7 @@ def user_authorized_keys(
 
         # And every public key is present
         for key in public_keys:
-            yield from files.line(path=authorized_key_file, line=key)
+            yield from files.line(path=authorized_key_file, line=key, ensure_newline=True)
 
 
 @operation
@@ -913,6 +953,7 @@ def user(
     comment=None,
     add_deploy_dir=True,
     unique=True,
+    password=None,
 ):
     """
     Add/remove/update system users & their ssh `authorized_keys`.
@@ -928,9 +969,11 @@ def user(
     + ensure_home: whether to ensure the ``home`` directory exists
     + create_home: whether to new user create home directories from the system skeleton
     + system: whether to create a system account
+    + uid: use a specific userid number
     + comment: the user GECOS comment
     + add_deploy_dir: any public_key filenames are relative to the deploy directory
     + unique: prevent creating users with duplicate UID
+    + password: set the encrypted password for the user
 
     Home directory:
         When ``ensure_home`` or ``public_keys`` are provided, ``home`` defaults to
@@ -970,7 +1013,7 @@ def user(
     users = host.get_fact(Users)
     existing_groups = host.get_fact(Groups)
     existing_user = users.get(user)
-
+    os_type = host.get_fact(Os)
     if groups is None:
         groups = []
 
@@ -982,7 +1025,10 @@ def user(
     # User not wanted?
     if not present:
         if existing_user:
-            yield "userdel {0}".format(user)
+            if os_type == "FreeBSD":
+                yield "pw userdel -n {0}".format(user)
+            else:
+                yield "userdel {0}".format(user)
             users.pop(user)
         return
 
@@ -1012,7 +1058,10 @@ def user(
             args.append("-r")
 
         if uid:
-            args.append("--uid {0}".format(uid))
+            if os_type == "FreeBSD":
+                args.append("-u {0}".format(uid))
+            else:
+                args.append("--uid {0}".format(uid))
 
         if comment:
             args.append("-c '{0}'".format(comment))
@@ -1022,22 +1071,39 @@ def user(
 
         if create_home:
             args.append("-m")
+        else:
+            args.append("-M")
+
+        if password:
+            args.append("-p '{0}'".format(password))
 
         # Users are often added by other operations (package installs), so check
         # for the user at runtime before adding.
-        yield "grep '^{1}:' /etc/passwd || useradd {0} {1}".format(
-            " ".join(args),
-            user,
-        )
+
+        add_user_command = "useradd"
+        if os_type == "FreeBSD":
+            add_user_command = "pw useradd"
+            yield "grep '^{2}:' /etc/passwd || {0} -n {2} {1}".format(
+                add_user_command,
+                " ".join(args),
+                user,
+            )
+        else:
+            yield "grep '^{2}:' /etc/passwd || {0} {1} {2}".format(
+                add_user_command,
+                " ".join(args),
+                user,
+            )
         users[user] = {
             "comment": comment,
             "home": home,
             "shell": shell,
             "group": group,
             "groups": groups,
+            "password": password,
         }
 
-    # User exists and we want them, check home/shell/keys
+    # User exists and we want them, check home/shell/keys/password
     else:
         args = []
 
@@ -1060,9 +1126,15 @@ def user(
         if comment and existing_user["comment"] != comment:
             args.append("-c '{0}'".format(comment))
 
+        if password and existing_user["password"] != password:
+            args.append("-p '{0}'".format(password))
+
         # Need to mod the user?
         if args:
-            yield "usermod {0} {1}".format(" ".join(args), user)
+            if os_type == "FreeBSD":
+                yield "pw usermod -n {1} {0}".format(" ".join(args), user)
+            else:
+                yield "usermod {0} {1}".format(" ".join(args), user)
             if comment:
                 existing_user["comment"] = comment
             if home:
@@ -1073,6 +1145,8 @@ def user(
                 existing_user["group"] = group
             if groups:
                 existing_user["groups"] = groups
+            if password:
+                existing_user["password"] = password
 
     # Ensure home directory ownership
     if ensure_home:
@@ -1094,3 +1168,110 @@ def user(
             authorized_key_directory="{0}/.ssh".format(home),
             authorized_key_filename=None,
         )
+
+
+@operation
+def locale(
+    locale,
+    present=True,
+):
+    """
+    Enable/Disable locale.
+
+    + locale: name of the locale to enable/disable
+    + present: whether this locale should be present or not
+
+    **Examples:**
+
+    .. code:: python
+
+        server.locale(
+            name="Ensure en_GB.UTF-8 locale is not present",
+            locale="en_GB.UTF-8",
+            present=False,
+        )
+
+        server.locale(
+            name="Ensure en_GB.UTF-8 locale is present",
+            locale="en_GB.UTF-8",
+        )
+
+    """
+
+    locales = host.get_fact(Locales)
+
+    logger.debug("Enabled locales: {0}".format(locales))
+
+    locales_definitions_file = "/etc/locale.gen"
+
+    # Find the matching line in /etc/locale.gen
+    matching_lines = host.get_fact(
+        FindInFile, path=locales_definitions_file, pattern=rf"^.*{locale}[[:space:]]\+.*$"
+    )
+
+    if not matching_lines:
+        raise OperationError(f"Locale {locale} not found in {locales_definitions_file}")
+
+    if len(matching_lines) > 1:
+        raise OperationError(f"Multiple locales matches for {locale} in {locales_definitions_file}")
+
+    matching_line = matching_lines[0]
+
+    # Remove locale
+    if not present and locale in locales:
+        logger.debug(f"Removing locale {locale}")
+
+        yield from files.line(
+            path=locales_definitions_file, line=f"^{matching_line}$", replace=f"# {matching_line}"
+        )
+
+        yield "locale-gen"
+
+    # Add locale
+    if present and locale not in locales:
+        logger.debug(f"Adding locale {locale}")
+
+        yield from files.replace(
+            path=locales_definitions_file,
+            text=f"^{matching_line}$",
+            replace=f"{matching_line}".replace("# ", ""),
+        )
+
+        yield "locale-gen"
+
+
+@operation
+def security_limit(
+    domain,
+    limit_type,
+    item,
+    value,
+):
+    """
+    Edit /etc/security/limits.conf configuration.
+
+    + domain: the domain (user, group, or wildcard) for the limit
+    + limit_type: the type of limit (hard or soft)
+    + item: the item to limit (e.g., nofile, nproc)
+    + value: the value for the limit
+
+    **Example:**
+
+    .. code:: python
+
+        security_limit(
+            name="Set nofile limit for all users",
+            domain='*',
+            limit_type='soft',
+            item='nofile',
+            value='1024',
+        )
+    """
+
+    line_format = f"{domain}\t{limit_type}\t{item}\t{value}"
+
+    yield from files.line(
+        path="/etc/security/limits.conf",
+        line=f"^{domain}[[:space:]]+{limit_type}[[:space:]]+{item}",
+        replace=line_format,
+    )
