@@ -1,17 +1,22 @@
 from collections import defaultdict
 from os import listdir, path
 from types import GeneratorType
+from typing import Any, Callable, Dict, List, Optional, Tuple, TypeVar, Union
 
 from pyinfra import logger
 from pyinfra.api.inventory import Inventory
 from pyinfra.context import ctx_inventory
-from pyinfra_cli.util import exec_file
+
+from .exceptions import CliError
+from .util import exec_file, try_import_module_attribute
+
+HostType = Union[str, Tuple[str, Dict]]
 
 # Hosts in an inventory can be just the hostname or a tuple (hostname, data)
 ALLOWED_HOST_TYPES = (str, tuple)
 
 
-def _is_inventory_group(key, value):
+def _is_inventory_group(key: str, value: Any):
     """
     Verify that a module-level variable (key = value) is a valid inventory group.
     """
@@ -30,11 +35,11 @@ def _is_inventory_group(key, value):
     return all(isinstance(item, ALLOWED_HOST_TYPES) for item in value)
 
 
-def _get_group_data(dirname):
+def _get_group_data(dirname: str):
     group_data = {}
     group_data_directory = path.join(dirname, "group_data")
 
-    logger.debug(f"Checking possible group_data directory: {dirname}")
+    logger.debug("Checking possible group_data directory: %s", dirname)
 
     if path.exists(group_data_directory):
         files = listdir(group_data_directory)
@@ -46,7 +51,7 @@ def _get_group_data(dirname):
             group_data_file = path.join(group_data_directory, file)
             group_name = path.basename(file)[:-3]
 
-            logger.debug("Looking for group data in: {0}".format(group_data_file))
+            logger.debug("Looking for group data in: %s", group_data_file)
 
             # Read the files locals into a dict
             attrs = exec_file(group_data_file, return_locals=True)
@@ -61,13 +66,96 @@ def _get_group_data(dirname):
     return group_data
 
 
-def _get_groups_from_filename(inventory_filename):
+def _get_groups_from_filename(inventory_filename: str):
     attrs = exec_file(inventory_filename, return_locals=True)
 
     return {key: value for key, value in attrs.items() if _is_inventory_group(key, value)}
 
 
-def make_inventory(inventory_filename, override_data=None, cwd=None, group_data_directories=None):
+T = TypeVar("T")
+
+
+def _get_any_tuple_first(item: Union[T, Tuple[T, Any]]) -> T:
+    return item[0] if isinstance(item, tuple) else item
+
+
+def make_inventory(
+    inventory: str,
+    override_data=None,
+    cwd: Optional[str] = None,
+    group_data_directories=None,
+):
+    # First, try loading the inventory as if it's a Python import function
+    try:
+        inventory_func = try_import_module_attribute(inventory)
+    except (CliError, ValueError):
+        # If not an import, load as if from the filesystem *or* comma separated list, which also
+        # loads any all.py group data files (imported functions do not load group data).
+        return make_inventory_from_files(inventory, override_data, cwd, group_data_directories)
+    else:
+        return make_inventory_from_func(inventory_func, override_data)
+
+
+def make_inventory_from_func(
+    inventory_func: Callable[[], Dict[str, List[HostType]]],
+    override_data: Optional[Dict[Any, Any]] = None,
+):
+    logger.warning("Loading inventory via import function is in alpha!")
+
+    try:
+        groups = inventory_func()
+    except Exception as e:
+        raise CliError(f"Failed to load inventory function: {inventory_func.__name__}: {e}")
+
+    if not isinstance(groups, dict):
+        raise TypeError(f"Inventory function {inventory_func.__name__} did not return a dictionary")
+
+    # TODO: this shouldn't be required to make an inventory, groups should suffice
+    combined_host_list = set()
+    groups_with_data: Dict[str, Tuple[List[HostType], Dict]] = {}
+
+    for key, hosts in groups.items():
+        data: Dict = {}
+
+        if isinstance(hosts, tuple):
+            hosts, data = hosts
+
+        if not isinstance(data, dict):
+            raise TypeError(
+                f"Inventory function {inventory_func.__name__} "
+                f"group contains non-dictionary data: {key}"
+            )
+
+        for host in hosts:
+            if not isinstance(host, ALLOWED_HOST_TYPES):
+                raise TypeError(
+                    f"Inventory function {inventory_func.__name__} invalid host: {host}"
+                )
+
+            host = _get_any_tuple_first(host)
+
+            if not isinstance(host, str):
+                raise TypeError(
+                    f"Inventory function {inventory_func.__name__} invalid host name: {host}"
+                )
+
+            combined_host_list.add(host)
+
+        groups_with_data[key] = (hosts, data)
+
+    return Inventory(
+        (list(combined_host_list), {}),
+        override_data=override_data,
+        **groups_with_data,
+    )
+
+
+def make_inventory_from_files(
+    inventory_filename: str,
+    override_data=None,
+    cwd: Optional[str] = None,
+    group_data_directories=None,
+):
     """
     Builds a ``pyinfra.api.Inventory`` from the filesystem. If the file does not exist
     and doesn't contain a / attempts to use that as the only hostname.
@@ -75,6 +163,9 @@ def make_inventory(inventory_filename, override_data=None, cwd=None, group_data_
 
     file_groupname = None
 
+    # TODO: this type is complex & convoluted, fix this
+    groups: Dict[str, Union[List[str], Tuple[List[str], Dict[str, Any]]]]
+
     # If we're not a valid file we assume a list of comma separated hostnames
     if not path.exists(inventory_filename):
         groups = {
@@ -86,7 +177,7 @@ def make_inventory(inventory_filename, override_data=None, cwd=None, group_data_
         # ie inventories/dev.py means all the hosts are in the dev group, if not present
         file_groupname = path.basename(inventory_filename).rsplit(".", 1)[0]
 
-    all_data = {}
+    all_data: Dict[str, Any] = {}
 
     if "all" in groups:
         all_hosts = groups.pop("all")
@@ -99,11 +190,11 @@ def make_inventory(inventory_filename, override_data=None, cwd=None, group_data_
         all_hosts = []
         for hosts in groups.values():
             # Groups can be a list of hosts or tuple of (hosts, data)
-            hosts = hosts[0] if isinstance(hosts, tuple) else hosts
+            hosts = _get_any_tuple_first(hosts)
 
             for host in hosts:
                 # Hosts can be a hostname or tuple of (hostname, data)
-                hostname = host[0] if isinstance(host, tuple) else host
+                hostname = _get_any_tuple_first(host)
 
                 if hostname not in all_hosts:
                     all_hosts.append(hostname)
@@ -128,7 +219,9 @@ def make_inventory(inventory_filename, override_data=None, cwd=None, group_data_
     }
     fake_inventory = Inventory((all_hosts, all_data), **fake_groups)
 
-    possible_group_data_folders = [cwd]
+    possible_group_data_folders = []
+    if cwd:
+        possible_group_data_folders.append(cwd)
     inventory_dirname = path.abspath(path.dirname(inventory_filename))
     if inventory_dirname != cwd:
         possible_group_data_folders.append(inventory_dirname)
@@ -136,7 +229,7 @@ def make_inventory(inventory_filename, override_data=None, cwd=None, group_data_
     if group_data_directories:
         possible_group_data_folders.extend(group_data_directories)
 
-    group_data = defaultdict(dict)
+    group_data: Dict[str, Dict[str, Any]] = defaultdict(dict)
 
     with ctx_inventory.use(fake_inventory):
         for folder in possible_group_data_folders:
@@ -162,7 +255,4 @@ def make_inventory(inventory_filename, override_data=None, cwd=None, group_data_
     for name, data in group_data.items():
         groups[name] = ([], data)
 
-    return (
-        Inventory(groups.pop("all"), override_data=override_data, **groups),
-        file_groupname and file_groupname.lower(),
-    )
+    return Inventory(groups.pop("all"), override_data=override_data, **groups)
