@@ -3,6 +3,7 @@ import sys
 import warnings
 from fnmatch import fnmatch
 from os import chdir as os_chdir, getcwd, path
+from typing import Iterable, List, Tuple, Union
 
 import click
 
@@ -11,12 +12,12 @@ from pyinfra.api import Config, State
 from pyinfra.api.connect import connect_all, disconnect_all
 from pyinfra.api.exceptions import NoGroupError, PyinfraError
 from pyinfra.api.facts import get_facts
-from pyinfra.api.operation import add_op
 from pyinfra.api.operations import run_ops
 from pyinfra.api.util import get_kwargs_str
 from pyinfra.context import ctx_config, ctx_inventory, ctx_state
 from pyinfra.operations import server
 
+from .commands import get_facts_and_args, get_func_and_args
 from .exceptions import CliError, UnexpectedExternalError, UnexpectedInternalError
 from .inventory import make_inventory
 from .log import setup_logging
@@ -29,13 +30,7 @@ from .prints import (
     print_state_operations,
     print_support_info,
 )
-from .util import (
-    exec_file,
-    get_facts_and_args,
-    get_operation_and_args,
-    load_deploy_file,
-    parse_cli_arg,
-)
+from .util import exec_file, load_deploy_file, load_func, parse_cli_arg
 from .virtualenv import init_virtualenv
 
 
@@ -74,7 +69,7 @@ def _print_support(ctx, param, value):
     help="Restrict the target hosts by name and group name.",
     multiple=True,
 )
-@click.option("--fail-percent", type=int, help="% of hosts allowed to fail.")
+@click.option("--fail-percent", type=int, help="% of hosts that need to fail before exiting early.")
 @click.option(
     "--data",
     multiple=True,
@@ -185,7 +180,7 @@ def cli(*args, **kwargs):
     pyinfra manages the state of one or more servers. It can be used for
     app/service deployment, config management and ad-hoc command execution.
 
-    Documentation: pyinfra.readthedocs.io
+    Documentation: docs.pyinfra.com
 
     # INVENTORY
 
@@ -245,49 +240,153 @@ def cli(*args, **kwargs):
             disconnect_all(state)
 
 
+class CliCommands:
+    DEBUG_INVENTORY = "DEBUG_INVENTORY"
+    FACT = "FACT"
+    SHELL = "SHELL"
+    DEPLOY_FILES = "DEPLOY_FILES"
+    FUNC = "FUNC"
+
+
 def _main(
     inventory,
-    operations,
-    verbosity,
-    chdir,
+    operations: Union[List, Tuple],
+    verbosity: int,
+    chdir: str,
     ssh_user,
-    ssh_port,
+    ssh_port: int,
     ssh_key,
-    ssh_key_password,
-    ssh_password,
-    winrm_username,
-    winrm_password,
+    ssh_key_password: str,
+    ssh_password: str,
+    winrm_username: str,
+    winrm_password: str,
     winrm_port,
     winrm_transport,
     shell_executable,
-    sudo,
-    sudo_user,
-    use_sudo_password,
-    su_user,
-    parallel,
-    fail_percent,
+    sudo: bool,
+    sudo_user: str,
+    use_sudo_password: bool,
+    su_user: str,
+    parallel: int,
+    fail_percent: int,
     data,
     group_data,
-    config_filename,
-    dry,
-    limit,
-    no_wait,
-    serial,
-    quiet,
-    debug,
-    debug_facts,
-    debug_operations,
-    support=None,
+    config_filename: str,
+    dry: bool,
+    limit: Iterable,
+    no_wait: bool,
+    serial: bool,
+    quiet: bool,
+    debug: bool,
+    debug_facts: bool,
+    debug_operations: bool,
+    support: bool = None,
 ):
     # Setup working directory
     #
-
     if chdir:
         os_chdir(chdir)
 
-    # Setup logging
+    # Setup logging & Bootstrap/Venv
     #
+    _setup_log_level(debug, quiet)
+    init_virtualenv()
 
+    # Check operations are valid and setup commands
+    #
+    original_operations, operations, command, chdir = _validate_operations(operations, chdir)
+
+    # Setup state, config & inventory
+    #
+    state = _setup_state(verbosity, quiet)
+    config = Config()
+    ctx_config.set(config)
+
+    # Update Config & Override Data
+    #
+    config = _set_config(
+        config,
+        config_filename,
+        sudo,
+        sudo_user,
+        use_sudo_password,
+        su_user,
+        parallel,
+        shell_executable,
+        fail_percent,
+        quiet,
+    )
+    override_data = _set_override_data(
+        data,
+        ssh_user,
+        ssh_key,
+        ssh_key_password,
+        ssh_port,
+        ssh_password,
+        winrm_username,
+        winrm_password,
+        winrm_port,
+        winrm_transport,
+    )
+
+    # Load up the inventory from the filesystem
+    #
+    echo_msg("--> Loading inventory...", quiet)
+    inventory = make_inventory(
+        inventory,
+        cwd=state.cwd,
+        override_data=override_data,
+        group_data_directories=group_data,
+    )
+    ctx_inventory.set(inventory)
+
+    # Now that we have inventory, apply --limit config override
+    initial_limit = _apply_inventory_limit(inventory, limit)
+
+    # Initialise the state
+    state.init(inventory, config, initial_limit=initial_limit)
+
+    if command == CliCommands.DEBUG_INVENTORY:
+        print_inventory(state)
+        _exit()
+
+    # Connect to the hosts & start handling the user commands
+    #
+    echo_msg("--> Connecting to hosts...", quiet)
+    connect_all(state)
+    state, config = _handle_commands(state, config, command, original_operations, operations, quiet)
+
+    # Print proposed changes, execute unless --dry, and exit
+    #
+    echo_msg("--> Proposed changes:", quiet)
+    print_meta(state)
+
+    # If --debug-facts or --debug-operations, print and exit
+    if debug_facts or debug_operations:
+        if debug_facts:
+            print_state_facts(state)
+
+        if debug_operations:
+            print_state_operations(state)
+
+        _exit()
+
+    if dry:
+        _exit()
+
+    echo_msg(quiet=quiet)
+    echo_msg("--> Beginning operation run...", quiet)
+
+    run_ops(state, serial=serial, no_wait=no_wait)
+
+    echo_msg("--> Results:", quiet)
+    print_results(state)
+    _exit()
+
+
+# Setup
+#
+def _setup_log_level(debug, quiet):
     if not debug and not sys.warnoptions:
         warnings.simplefilter("ignore")
 
@@ -299,32 +398,29 @@ def _main(
 
     setup_logging(log_level)
 
-    # Bootstrap any virtualenv
-    init_virtualenv()
 
-    #  Check operations are valid and setup command
-    #
+def _validate_operations(operations, chdir):
 
     # Make a copy before we overwrite
     original_operations = operations
 
     # Debug (print) inventory + group data
     if operations[0] == "debug-inventory":
-        command = "debug-inventory"
+        command = CliCommands.DEBUG_INVENTORY
 
     # Get one or more facts
     elif operations[0] == "fact":
-        command = "fact"
+        command = CliCommands.FACT
         operations = get_facts_and_args(operations[1:])
 
     # Execute a raw command with server.shell
     elif operations[0] == "exec":
-        command = "exec"
+        command = CliCommands.SHELL
         operations = operations[1:]
 
     # Execute one or more deploy files
     elif all(cmd.endswith(".py") for cmd in operations):
-        command = "deploy"
+        command = CliCommands.DEPLOY_FILES
 
         filenames = []
 
@@ -350,10 +446,10 @@ def _main(
 
         operations = filenames
 
-    # Operation w/optional args (<module>.<op> ARG1 ARG2 ...)
-    elif len(operations[0].split(".")) == 2:
-        command = "op"
-        operations = get_operation_and_args(operations)
+    # Load a function (op or deploy) directly with arguments
+    elif "." in operations[0]:
+        command = CliCommands.FUNC
+        operations = get_func_and_args(operations)
 
     else:
         raise CliError(
@@ -368,17 +464,10 @@ def _main(
             ),
         )
 
-    # Setup state, config & inventory
-    #
+    return original_operations, operations, command, chdir
 
-    cwd = getcwd()
-    if cwd not in sys.path:  # ensure cwd is present in sys.path
-        sys.path.append(cwd)
-
-    state = State()
-    state.cwd = cwd
-    ctx_state.set(state)
 
+def _set_verbosity(state, verbosity, quiet):
     if verbosity > 0:
         state.print_fact_info = True
         state.print_noop_info = True
@@ -389,11 +478,35 @@ def _main(
     if verbosity > 2:
         state.print_output = state.print_fact_output = True
 
-    if not quiet:
-        click.echo("--> Loading config...", err=True)
+    return state
 
-    config = Config()
-    ctx_config.set(config)
+
+def _setup_state(verbosity, quiet):
+    cwd = getcwd()
+    if cwd not in sys.path:  # ensure cwd is present in sys.path
+        sys.path.append(cwd)
+
+    state = State()
+    state.cwd = cwd
+    ctx_state.set(state)
+
+    state = _set_verbosity(state, verbosity, quiet)
+    return state
+
+
+def _set_config(
+    config,
+    config_filename,
+    sudo,
+    sudo_user,
+    use_sudo_password,
+    su_user,
+    parallel,
+    shell_executable,
+    fail_percent,
+    quiet,
+):
+    echo_msg("--> Loading config...", quiet)
 
     # Load up any config.py from the filesystem
     config_filename = path.join(state.cwd, config_filename)
@@ -425,9 +538,21 @@ def _main(
     if fail_percent is not None:
         config.FAIL_PERCENT = fail_percent
 
-    if not quiet:
-        click.echo("--> Loading inventory...", err=True)
+    return config
 
+
+def _set_override_data(
+    data,
+    ssh_user,
+    ssh_key,
+    ssh_key_password,
+    ssh_port,
+    ssh_password,
+    winrm_username,
+    winrm_password,
+    winrm_port,
+    winrm_transport,
+):
     override_data = {}
 
     for arg in data:
@@ -450,16 +575,10 @@ def _main(
         if value:
             override_data[key] = value
 
-    # Load up the inventory from the filesystem
-    inventory, inventory_group = make_inventory(
-        inventory,
-        cwd=state.cwd,
-        override_data=override_data,
-        group_data_directories=group_data,
-    )
-    ctx_inventory.set(inventory)
+    return override_data
 
-    # Now that we have inventory, apply --limit config override
+
+def _apply_inventory_limit(inventory, limit):
     initial_limit = None
     if limit:
         all_limit_hosts = []
@@ -476,131 +595,101 @@ def _main(
             all_limit_hosts.extend(limit_hosts)
         initial_limit = list(set(all_limit_hosts))
 
-    # Initialise the state
-    state.init(inventory, config, initial_limit=initial_limit)
+    return initial_limit
 
-    if command == "debug-inventory":
-        print_inventory(state)
-        _exit()
 
-    # Connect to the hosts & start handling the user commands
-    #
-
-    if not quiet:
-        click.echo(err=True)
-        click.echo("--> Connecting to hosts...", err=True)
-
-    connect_all(state)
-
-    if command == "fact":
-        if not quiet:
-            click.echo(err=True)
-            click.echo("--> Gathering facts...", err=True)
-
-        state.print_fact_info = True
-        fact_data = {}
-
-        for i, command in enumerate(operations):
-            fact_cls, args, kwargs = command
-            fact_key = fact_cls.name
-
-            if args or kwargs:
-                fact_key = "{0}{1}{2}".format(
-                    fact_cls.name,
-                    args or "",
-                    " ({0})".format(get_kwargs_str(kwargs)) if kwargs else "",
-                )
-
-            try:
-                fact_data[fact_key] = get_facts(
-                    state,
-                    fact_cls,
-                    args=args,
-                    kwargs=kwargs,
-                    apply_failed_hosts=False,
-                )
-            except PyinfraError:
-                pass
+# Operations Execution
+#
+def _handle_commands(state, config, command, original_operations, operations, quiet):
 
+    if command is CliCommands.FACT:
+        state, fact_data = _run_fact_operations(state, config, operations, quiet)
         print_facts(fact_data)
         _exit()
 
-    if command == "exec":
-        state.print_output = True
-        add_op(
-            state,
-            server.shell,
-            " ".join(operations),
-            _allow_cli_mode=True,
-        )
+    if command == CliCommands.SHELL:
+        state = _run_exec_operations(state, config, operations, quiet)
 
-    elif command == "deploy":
-        if not quiet:
-            click.echo(err=True)
-            click.echo("--> Preparing operations...", err=True)
+    elif command == CliCommands.DEPLOY_FILES:
+        state, config, operations = _run_deploy_operations(state, config, operations, quiet)
 
-        # Number of "steps" to make = number of files * number of hosts
-        for i, filename in enumerate(operations):
-            logger.info("Loading: {0}".format(click.style(filename, bold=True)))
+    elif command == CliCommands.FUNC:
+        state, kwargs = _run_func_operations(state, config, operations, original_operations, quiet)
 
-            state.current_op_file_number = i
-            load_deploy_file(state, filename)
+    return state, config
 
-            # Remove any config changes introduced by the deploy file & any includes
-            config.reset_locked_state()
 
-    elif command == "op":
-        if not quiet:
-            click.echo(err=True)
-            click.echo("--> Preparing operation...", err=True)
+def _run_fact_operations(state, config, operations, quiet):
+    echo_msg("--> Gathering facts...", quiet)
 
-        op, args = operations
-        args, kwargs = args
-        kwargs["_allow_cli_mode"] = True
+    state.print_fact_info = True
+    fact_data = {}
 
-        def print_host_ready(host):
-            logger.info(
-                "{0}{1} {2}".format(
-                    host.print_prefix,
-                    click.style("Ready:", "green"),
-                    click.style(original_operations[0], bold=True),
-                ),
+    for i, command in enumerate(operations):
+        fact_cls, args, kwargs = command
+        fact_key = fact_cls.name
+
+        if args or kwargs:
+            _fact_args = args or ""
+            _fact_details = " ({0})".format(get_kwargs_str(kwargs)) if kwargs else ""
+            fact_key = "{0}{1}{2}".format(fact_cls.name, _fact_args, _fact_details)
+
+        try:
+            fact_data[fact_key] = get_facts(
+                state,
+                fact_cls,
+                args=args,
+                kwargs=kwargs,
+                apply_failed_hosts=False,
             )
+        except PyinfraError:
+            pass
 
-        kwargs["_after_host_callback"] = print_host_ready
+    return state, fact_data
 
-        add_op(state, op, *args, **kwargs)
 
-    # Print proposed changes, execute unless --dry, and exit
-    #
+def _run_exec_operations(state, config, operations, quiet):
+    state.print_output = True
+    load_func(
+        state,
+        server.shell,
+        " ".join(operations),
+    )
+    return state
 
+
+def _run_deploy_operations(state, config, operations, quiet):
+    echo_msg("--> Preparing Operations...", quiet)
+
+    # Number of "steps" to make = number of files * number of hosts
+    for i, filename in enumerate(operations):
+        _log_styled_msg = click.style(filename, bold=True)
+        logger.info("Loading: {0}".format(_log_styled_msg))
+
+        state.current_op_file_number = i
+        load_deploy_file(state, filename)
+
+        # Remove any config changes introduced by the deploy file & any includes
+        config.reset_locked_state()
+
+    return state, config, operations
+
+
+def _run_func_operations(state, config, operations, original_operations, quiet):
+    echo_msg("--> Preparing operation...", quiet)
+
+    op, args = operations
+    args, kwargs = args
+
+    load_func(state, op, *args, **kwargs)
+
+    return state, kwargs
+
+
+# Utils
+#
+def echo_msg(msg=None, quiet=None):
     if not quiet:
         click.echo(err=True)
-        click.echo("--> Proposed changes:", err=True)
-    print_meta(state)
-
-    # If --debug-facts or --debug-operations, print and exit
-    if debug_facts or debug_operations:
-        if debug_facts:
-            print_state_facts(state)
-
-        if debug_operations:
-            print_state_operations(state)
-
-        _exit()
-
-    if dry:
-        _exit()
-
-    if not quiet:
-        click.echo(err=True)
-
-    if not quiet:
-        click.echo("--> Beginning operation run...", err=True)
-    run_ops(state, serial=serial, no_wait=no_wait)
-
-    if not quiet:
-        click.echo("--> Results:", err=True)
-    print_results(state)
-
-    _exit()
+    if msg:
+        click.echo(msg, err=True)
