@@ -2,6 +2,7 @@ import json
 import platform
 import re
 import sys
+from typing import TYPE_CHECKING, Callable, Dict, Iterator, List, Tuple, Union
 
 import click
 
@@ -10,6 +11,10 @@ from pyinfra.api.host import Host
 
 from .util import json_encode
 
+if TYPE_CHECKING:
+    from pyinfra.api.state import State
+
+
 ANSI_RE = re.compile(r"\033\[((?:\d|;)*)([a-zA-Z])")
 
 
@@ -17,8 +22,8 @@ def _strip_ansi(value):
     return ANSI_RE.sub("", value)
 
 
-def _get_group_combinations(inventory):
-    group_combinations = {}
+def _get_group_combinations(inventory: Iterator[Host]):
+    group_combinations: Dict[Tuple, List[Host]] = {}
 
     for host in inventory:
         # Tuple for hashability, set to normalise order
@@ -45,13 +50,13 @@ def jsonify(data, *args, **kwargs):
     return json.dumps(data, *args, **kwargs)
 
 
-def print_state_facts(state):
+def print_state_facts(state: "State"):
     click.echo(err=True)
     click.echo("--> Facts:", err=True)
     click.echo(jsonify(state.facts, indent=4, default=json_encode), err=True)
 
 
-def print_state_operations(state):
+def print_state_operations(state: "State"):
     state_ops = {host: ops for host, ops in state.ops.items() if state.is_host_in_limit(host)}
 
     click.echo(err=True)
@@ -108,7 +113,7 @@ def print_fact(fact_data):
     click.echo(jsonify(fact_data, indent=4, default=json_encode), err=True)
 
 
-def print_inventory(state):
+def print_inventory(state: "State"):
     for host in state.inventory:
         click.echo(err=True)
         click.echo(host.print_prefix, err=True)
@@ -197,9 +202,9 @@ def print_rows(rows):
         func(line)
 
 
-def print_meta(state):
+def print_meta(state: "State"):
     group_combinations = _get_group_combinations(state.inventory.iter_activated_hosts())
-    rows = []
+    rows: List[Tuple[Callable, Union[List[str], str]]] = []
 
     for i, (groups, hosts) in enumerate(group_combinations.items(), 1):
         if not hosts:
@@ -251,9 +256,9 @@ def print_meta(state):
     print_rows(rows)
 
 
-def print_results(state):
+def print_results(state: "State"):
     group_combinations = _get_group_combinations(state.inventory.iter_activated_hosts())
-    rows = []
+    rows: List[Tuple[Callable, Union[List[str], str]]] = []
 
     for i, (groups, hosts) in enumerate(group_combinations.items(), 1):
         if not hosts:
@@ -290,7 +295,8 @@ def print_results(state):
             meta = state.meta[host]
             success_ops = results["success_ops"]
             partial_ops = results["partial_ops"]
-            changed_ops = success_ops - meta["ops_no_change"]
+            # TODO: type meta object
+            changed_ops = success_ops - meta["ops_no_change"]  # type: ignore
             error_ops = results["error_ops"]
             ignored_error_ops = results["ignored_error_ops"]
 
@@ -308,11 +314,11 @@ def print_results(state):
                 host_args = ("red",)
                 host_kwargs["bold"] = True
 
-            changed_str = "Changed: {0}".format(click.style(changed_ops, bold=True))
+            changed_str = "Changed: {0}".format(click.style(f"{changed_ops}", bold=True))
             if partial_ops:
                 changed_str = f"{changed_str} ({partial_ops} partial)"
 
-            error_str = "Errors: {0}".format(click.style(error_ops, bold=True))
+            error_str = "Errors: {0}".format(click.style(f"{error_ops}", bold=True))
             if ignored_error_ops:
                 error_str = f"{error_str} ({ignored_error_ops} ignored)"
 
@@ -322,7 +328,7 @@ def print_results(state):
                     [
                         host.style_print_prefix(*host_args, **host_kwargs),
                         changed_str,
-                        "No change: {0}".format(click.style(meta["ops_no_change"], bold=True)),
+                        "No change: {0}".format(click.style(f"{meta['ops_no_change']}", bold=True)),
                         error_str,
                     ],
                 ),
