@@ -2,23 +2,11 @@ import json
 import os
 from datetime import datetime
 from importlib import import_module
+from io import IOBase
 from os import path
 from pathlib import Path
 from types import FunctionType, ModuleType
-
-# py2/3 switcheroo
-try:
-    from types import FileType
-
-    from cStringIO import InputType, OutputType
-    from StringIO import StringIO
-
-    io_bases = (FileType, OutputType, InputType, StringIO)
-
-except ImportError:  # pragma: no cover
-    from io import IOBase
-
-    io_bases = IOBase
+from typing import TYPE_CHECKING, Callable
 
 import click
 import gevent
@@ -33,6 +21,9 @@ from pyinfra.progress import progress_spinner
 
 from .exceptions import CliError, UnexpectedExternalError
 
+if TYPE_CHECKING:
+    from pyinfra.api.state import State
+
 # Cache for compiled Python deploy code
 PYTHON_CODES = {}
 
@@ -44,7 +35,7 @@ def is_subdir(child, parent):
     return not relative.startswith(os.pardir)
 
 
-def exec_file(filename, return_locals=False, is_deploy_code=False):
+def exec_file(filename, return_locals: bool = False, is_deploy_code: bool = False):
     """
     Execute a Python file and optionally return it's attributes as a dict.
     """
@@ -53,7 +44,7 @@ def exec_file(filename, return_locals=False, is_deploy_code=False):
     state.current_exec_filename = filename
 
     if filename not in PYTHON_CODES:
-        with open(filename, "r") as f:
+        with open(filename, "r", encoding="utf-8") as f:
             code = f.read()
 
         code = compile(code, filename, "exec")
@@ -68,7 +59,7 @@ def exec_file(filename, return_locals=False, is_deploy_code=False):
     try:
         exec(PYTHON_CODES[filename], data)
     except Exception as e:
-        if isinstance(e, PyinfraError):
+        if isinstance(e, (PyinfraError, UnexpectedExternalError)):
             raise
         raise UnexpectedExternalError(e, filename)
 
@@ -97,7 +88,7 @@ def json_encode(obj):
     if isinstance(obj, datetime):
         return obj.isoformat()
 
-    if isinstance(obj, io_bases):
+    if isinstance(obj, IOBase):
         if hasattr(obj, "name"):
             return "File: {0}".format(obj.name)
 
@@ -142,104 +133,47 @@ def parse_cli_arg(arg):
     return arg
 
 
-def get_operation_and_args(commands):
-    operation_name = commands[0]
+def try_import_module_attribute(path, prefix=None):
+    mod_path, attr_name = path.rsplit(".", 1)
+    module = None
 
-    # Get the module & operation name
-    op_module, op_name = operation_name.rsplit(".", 1)
-
-    # Try to load the requested operation from the main operations package.
-    # If that fails, try to load from the user's operations package.
-    try:
-        op_module = import_module("pyinfra.operations.{0}".format(op_module))
-    except ImportError:
+    if prefix:
+        full_path = f"{prefix}.{mod_path}"
         try:
-            op_module = import_module(str(op_module))
-        except ImportError:
-            raise CliError("No such module: {0}".format(op_module))
-
-    op = getattr(op_module, op_name, None)
-    if not op:
-        raise CliError("No such operation: {0}".format(operation_name))
-
-    # Parse the arguments
-    operation_args = commands[1:]
-
-    if len(operation_args) == 1:
-        # Check if we're JSON (in which case we expect a list of two items:
-        # a list of args and a dict of kwargs).
-        try:
-            args, kwargs = json.loads(operation_args[0])
-            return op, (args or (), kwargs or {})
-        except ValueError:
+            module = import_module(full_path)
+        except (ModuleNotFoundError, ImportError):
             pass
+    else:
+        full_path = mod_path
 
-    args = [parse_cli_arg(arg) for arg in operation_args if "=" not in arg]
-
-    kwargs = {
-        key: parse_cli_arg(value)
-        for key, value in [arg.split("=", 1) for arg in operation_args if "=" in arg]
-    }
-
-    return op, (args, kwargs)
-
-
-def get_facts_and_args(commands):
-    facts = []
-
-    current_fact = None
-
-    for command in commands:
-        if "=" in command:
-            if not current_fact:
-                raise CliError("Invalid fact commands: `{0}`".format(commands))
-
-            key, value = command.split("=", 1)
-            current_fact[2][key] = value
-            continue
-
-        if current_fact:
-            facts.append(current_fact)
-            current_fact = None
-
-        if "." not in command:
-            raise CliError(f"Invalid fact: `{command}`, should be in the format `module.cls`")
-
-        fact_module, fact_name = command.rsplit(".", 1)
+    if module is None:
         try:
-            fact_module = import_module("pyinfra.facts.{0}".format(fact_module))
-        except ImportError:
-            try:
-                fact_module = import_module(str(fact_module))
-            except ImportError:
-                raise CliError("No such module: `{0}`".format(fact_module))
+            module = import_module(mod_path)
+        except (ModuleNotFoundError, ImportError):
+            raise CliError(f"No such module: {full_path}")
 
-        fact_cls = getattr(fact_module, fact_name, None)
-        if not fact_cls:
-            raise CliError("No such fact: `{0}`".format(command))
+    attr = getattr(module, attr_name, None)
+    if attr is None:
+        raise CliError(f"No such attribute in module {full_path}: {attr_name}")
 
-        current_fact = (fact_cls, (), {})
-
-    if current_fact:
-        facts.append(current_fact)
-
-    return facts
+    return attr
 
 
-def load_deploy_file(state, filename):
-    state.current_deploy_filename = filename
-
+def _parallel_load_hosts(state: "State", callback: Callable, name: str):
     def load_file(local_host):
-        with ctx_config.use(state.config.copy()):
-            with ctx_host.use(local_host):
-                exec_file(filename)
-                logger.info(
-                    "{0}{1} {2}".format(
-                        local_host.print_prefix,
-                        click.style("Ready:", "green"),
-                        click.style(filename, bold=True),
-                    ),
-                )
+        try:
+            with ctx_config.use(state.config.copy()):
+                with ctx_host.use(local_host):
+                    callback()
+                    logger.info(
+                        "{0}{1} {2}".format(
+                            local_host.print_prefix,
+                            click.style("Ready:", "green"),
+                            click.style(name, bold=True),
+                        ),
+                    )
+        except Exception as e:
+            return e
 
     greenlet_to_host = {
         state.pool.spawn(load_file, host): host for host in state.inventory.iter_active_hosts()
@@ -248,5 +182,16 @@ def load_deploy_file(state, filename):
     with progress_spinner(greenlet_to_host.values()) as progress:
         for greenlet in gevent.iwait(greenlet_to_host.keys()):
             host = greenlet_to_host[greenlet]
-            greenlet.get()
+            result = greenlet.get()
+            if isinstance(result, Exception):
+                raise result
             progress(host)
+
+
+def load_deploy_file(state: "State", filename):
+    state.current_deploy_filename = filename
+    _parallel_load_hosts(state, lambda: exec_file(filename), filename)
+
+
+def load_func(state: "State", op_func, *args, **kwargs):
+    _parallel_load_hosts(state, lambda: op_func(*args, **kwargs), op_func.__name__)
